@@ -30,9 +30,9 @@
 /* pplayers_at_war() thinks no contacts equals war, which often is
  * very annoying. */
 #define WAR(plr1, plr2) \
-  (plr1->diplstates[plr2->player_no].type == DS_WAR)
+  (plr1->diplstates[player_index(plr2)].type == DS_WAR)
 #define NEVER_MET(plr1, plr2) \
-  (plr1->diplstates[plr2->player_no].type == DS_NO_CONTACT)
+  (plr1->diplstates[player_index(plr2)].type == DS_NO_CONTACT)
 #define DEFENCE_POWER(punit) \
  (unit_type(punit)->defense_strength * unit_type(punit)->hp \
   * unit_type(punit)->firepower)
@@ -44,13 +44,14 @@
         > unit_type(punit)->transport_capacity)
 #define HOSTILE_PLAYER(pplayer, ai, aplayer) \
   (WAR(pplayer, aplayer)         \
-   || ai->diplomacy.player_intel[aplayer->player_no].countdown >= 0)
+   || ai->diplomacy.player_intel[player_index(aplayer)].countdown >= 0)
 #define UNITTYPE_COSTS(ut)						\
   (ut->pop_cost * 3 + ut->happy_cost					\
    + ut->upkeep[O_SHIELD] + ut->upkeep[O_FOOD] + ut->upkeep[O_GOLD])
 
-struct ai_choice;
-struct pf_path;
+/* Invasion types */
+#define INVASION_OCCUPY  0
+#define INVASION_ATTACK  1
 
 extern struct unit_type *simple_ai_types[U_LAST];
 
@@ -94,6 +95,9 @@ bool is_on_unit_upgrade_path(const struct unit_type *test,
 
 void update_simple_ai_types(void);
 
+/* Call this after rulesets are loaded */
+void unit_class_ai_init(void);
+
 #define simple_ai_unit_type_iterate(_ut)				\
 {									\
   struct unit_type *_ut;						\
