@@ -17,15 +17,18 @@
 
 #include <assert.h>
 
+/* utility */
 #include "dataio.h"
 #include "fcintl.h"
-#include "game.h"
 #include "hash.h"
 #include "log.h"
 #include "mem.h"
+
+/* common */
 #include "packets.h"
 
-#include "clinet.h"
+/* client */
+#include "client_main.h"
 
 #include "attribute.h"
 
@@ -321,9 +324,9 @@ static enum attribute_serial unserialize_hash( struct hash_table *hash,
 *****************************************************************************/
 void attribute_flush(void)
 {
-  struct player *pplayer = game.player_ptr;
+  struct player *pplayer = client.conn.playing;
 
-  if (!pplayer) {
+  if (!pplayer || client_is_observer() || !pplayer->is_alive) {
     return;
   }
 
@@ -339,7 +342,7 @@ void attribute_flush(void)
 
   serialize_hash(attribute_hash, &(pplayer->attribute_block.data),
 		 &(pplayer->attribute_block.length));
-  send_attribute_block(pplayer, &aconnection);
+  send_attribute_block(pplayer, &client.conn);
 }
 
 /****************************************************************************
@@ -348,7 +351,7 @@ void attribute_flush(void)
 *****************************************************************************/
 void attribute_restore(void)
 {
-  struct player *pplayer = game.player_ptr;
+  struct player *pplayer = client.conn.playing;
 
   if (!pplayer) {
     return;
