@@ -13,38 +13,16 @@
 #ifndef FC__CLINET_H
 #define FC__CLINET_H
 
-#define DEFAULT_SOCK_PORT 5555
-#define METALIST_ADDR "http://meta.freeciv.org/metaserver/"
+int connect_to_server(const char *username, const char *hostname, int port,
+		      char *errbuf, int errbufsize);
 
-struct connection;
+void make_connection(int socket, const char *username);
 
-int connect_to_server(char *name, char *hostname, int port,
-		      char *errbuf, int n_errbuf);
 void input_from_server(int fd);
+void input_from_server_till_request_got_processed(int fd,
+						  int expected_request_id);
 void disconnect_from_server(void);
 
-extern struct connection aconnection;
-/* this is the client's connection to the server */
-
-struct server
-{
-  char *name;
-  char *port;
-  char *version;
-  char *status;
-  char *players;
-  char *metastring;
-};
-
-#define SPECLIST_TAG server
-#define SPECLIST_TYPE struct server
-#include "speclist.h"
-
-#define server_list_iterate(serverlist, pserver) \
-  TYPED_LIST_ITERATE(struct server, serverlist, pserver)
-#define server_list_iterate_end  LIST_ITERATE_END
-
-struct server_list *create_server_list(char *errbuf, int n_errbuf);
-void delete_server_list(struct server_list *server_list);
-
+double try_to_autoconnect(void);
+void start_autoconnecting_to_server(void);
 #endif  /* FC__CLINET_H */
