@@ -37,18 +37,22 @@ Freeciv - Copyright (C) 2004 - The Freeciv Project
 #include <sys/wait.h>
 #endif
 
+/* common & utility */
 #include "capability.h"
+#include "fciconv.h"
 #include "fcintl.h"
+#include "ioz.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
+#include "rand.h"
 #include "registry.h"
-#include "shared.h"
 #include "support.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
+#include "clinet.h"		/* connect_to_server() */
 #include "packhand_gen.h"
 
 #include "chatline_common.h"
@@ -67,22 +71,11 @@ HANDLE loghandle = INVALID_HANDLE_VALUE;
 static pid_t server_pid = - 1;
 #endif
 
-char leader_name[MAX_LEN_NAME];
-char *current_filename = NULL;
-
 static char challenge_fullname[MAX_LEN_PATH];
 static bool client_has_hack = FALSE;
 
 int internal_server_port;
 
-const char *skill_level_names[NUM_SKILL_LEVELS] = { 
-  N_("novice"),
-  N_("easy"), 
-  N_("normal"), 
-  N_("hard")
- ,N_("experimental")
-};
-
 /************************************************************************** 
 The general chain of events:
 
@@ -138,7 +131,7 @@ bool can_client_access_hack(void)
 void client_kill_server(bool force)
 {
   if (is_server_running()) {
-    if (aconnection.used) {
+    if (client.conn.used) {
       /* This does a "soft" shutdown of the server by sending a /quit.
        *
        * This is useful when closing the client or disconnecting because it
@@ -190,7 +183,7 @@ bool client_start_server(void)
 #if !defined(HAVE_WORKING_FORK) && !defined(WIN32_NATIVE)
   /* Can't do much without fork */
   return FALSE;
-#else
+#else /* HAVE_WORKING_FORK || WIN32_NATIVE */
   char buf[512];
   int connect_tries = 0;
 # ifdef WIN32_NATIVE
@@ -204,13 +197,14 @@ bool client_start_server(void)
   char cmdline3[512];
   char logcmdline[512];
   char scriptcmdline[512];
-# endif
+  char savescmdline[512];
+# endif /* WIN32_NATIVE */
 
   /* only one server (forked from this client) shall be running at a time */
   /* This also resets client_has_hack. */
   client_kill_server(TRUE);
   
-  append_output_window(_("Starting server..."));
+  output_window_append(ftc_client, _("Starting server..."));
 
   /* find a free port */ 
   internal_server_port = find_next_free_port(DEFAULT_SOCK_PORT);
@@ -220,14 +214,14 @@ bool client_start_server(void)
   
   if (server_pid == 0) {
     int fd, argc = 0;
-    const int max_nargs = 13;
+    const int max_nargs = 17;
     char *argv[max_nargs + 1], port_buf[32];
 
     /* inside the child */
 
     /* Set up the command-line parameters. */
     my_snprintf(port_buf, sizeof(port_buf), "%d", internal_server_port);
-    argv[argc++] = "civserver";
+    argv[argc++] = "freeciv-server";
     argv[argc++] = "-p";
     argv[argc++] = port_buf;
     argv[argc++] = "-q";
@@ -235,6 +229,8 @@ bool client_start_server(void)
     argv[argc++] = "-e";
     argv[argc++] = "--saves";
     argv[argc++] = "~/.freeciv/saves";
+    argv[argc++] = "--scenarios";
+    argv[argc++] = "~/.freeciv/scenarios";
     if (logfile) {
       argv[argc++] = "--debug";
       argv[argc++] = "3";
@@ -266,7 +262,7 @@ bool client_start_server(void)
     }
 
     /* If it's still attatched to our terminal, things get messed up, 
-      but civserver needs *something* */ 
+      but freeciv-server needs *something* */ 
     fclose(stdin);
     fd = open("/dev/null", O_RDONLY);
     if (fd != 0) {
@@ -275,15 +271,15 @@ bool client_start_server(void)
 
     /* these won't return on success */ 
     execvp("./ser", argv);
-    execvp("./server/civserver", argv);
-    execvp("civserver", argv);
+    execvp("./server/freeciv-server", argv);
+    execvp("freeciv-server", argv);
     
-    /* This line is only reached if civserver cannot be started, 
+    /* This line is only reached if freeciv-server cannot be started, 
      * so we kill the forked process.
      * Calling exit here is dangerous due to X11 problems (async replies) */ 
     _exit(1);
   } 
-# else
+# else /* HAVE_WORKING_FORK */
 #  ifdef WIN32_NATIVE
   if (logfile) {
     loghandle = CreateFile(logfile, GENERIC_WRITE,
@@ -303,22 +299,30 @@ bool client_start_server(void)
   logcmdline[0] = 0;
   scriptcmdline[0] = 0;
 
+  /* the server expects command line arguments to be in local encoding */ 
   if (logfile) {
+    char *logfile_in_local_encoding = internal_to_local_string_malloc(logfile);
     my_snprintf(logcmdline, sizeof(logcmdline), " --debug 3 --log %s",
-		logfile);
+		logfile_in_local_encoding);
+    free(logfile_in_local_encoding);
   }
   if (scriptfile) {
+    char *scriptfile_in_local_encoding = internal_to_local_string_malloc(scriptfile);
     my_snprintf(scriptcmdline, sizeof(scriptcmdline),  " --read %s",
-		scriptfile);
+		scriptfile_in_local_encoding);
+    free(scriptfile_in_local_encoding);
   }
 
   interpret_tilde(savesdir, sizeof(savesdir), "~/.freeciv/saves");
+  internal_to_local_string_buffer(savesdir, savescmdline, sizeof(savescmdline));
 
   my_snprintf(options, sizeof(options), "-p %d -q 1 -e%s%s --saves \"%s\"",
-	      internal_server_port, logcmdline, scriptcmdline, savesdir);
+	      internal_server_port, logcmdline, scriptcmdline, savescmdline);
   my_snprintf(cmdline1, sizeof(cmdline1), "./ser %s", options);
-  my_snprintf(cmdline2, sizeof(cmdline2), "./server/civserver %s", options);
-  my_snprintf(cmdline3, sizeof(cmdline3), "civserver %s", options);
+  my_snprintf(cmdline2, sizeof(cmdline2),
+              "./server/freeciv-server %s", options);
+  my_snprintf(cmdline3, sizeof(cmdline3),
+              "freeciv-server %s", options);
 
   if (!CreateProcess(NULL, cmdline1, NULL, NULL, TRUE,
 		     DETACHED_PROCESS | NORMAL_PRIORITY_CLASS,
@@ -329,18 +333,19 @@ bool client_start_server(void)
       && !CreateProcess(NULL, cmdline3, NULL, NULL, TRUE,
 			DETACHED_PROCESS | NORMAL_PRIORITY_CLASS,
 			NULL, NULL, &si, &pi)) {
-    append_output_window(_("Couldn't start the server."));
-    append_output_window(_("You'll have to " "start one manually. Sorry..."));
+    output_window_append(ftc_client, _("Couldn't start the server."));
+    output_window_append(ftc_client,
+                         _("You'll have to start one manually. Sorry..."));
     return FALSE;
   }
   
   server_process = pi.hProcess;
 
-#  endif
-# endif
+#  endif /* WIN32_NATIVE */
+# endif /* HAVE_WORKING_FORK */
  
   /* a reasonable number of tries */ 
-  while (connect_to_server(user_name, "localhost", internal_server_port, 
+  while (connect_to_server(user_name, "127.0.0.1", internal_server_port, 
                            buf, sizeof(buf)) == -1) {
     myusleep(WAIT_BETWEEN_TRIES);
 #ifdef HAVE_WORKING_FORK
@@ -348,8 +353,8 @@ bool client_start_server(void)
     if (waitpid(server_pid, NULL, WNOHANG) != 0) {
       break;
     }
-#endif
-#endif
+#endif /* WIN32_NATIVE */
+#endif /* HAVE_WORKING_FORK */
     if (connect_tries++ > NUMBER_OF_TRIES) {
       break;
     }
@@ -357,13 +362,15 @@ bool client_start_server(void)
 
   /* weird, but could happen, if server doesn't support new startup stuff
    * capabilities won't help us here... */ 
-  if (!aconnection.used) {
+  if (!client.conn.used) {
     /* possible that server is still running. kill it */ 
     client_kill_server(TRUE);
 
-    append_output_window(_("Couldn't connect to the server."));
-    append_output_window(_("We probably couldn't start it from here."));
-    append_output_window(_("You'll have to start one manually. Sorry..."));
+    output_window_append(ftc_client, _("Couldn't connect to the server."));
+    output_window_append(ftc_client,
+                         _("We probably couldn't start it from here."));
+    output_window_append(ftc_client,
+                         _("You'll have to start one manually. Sorry..."));
     return FALSE;
   }
 
@@ -382,17 +389,39 @@ bool client_start_server(void)
    * Setting the option here is a bit of a hack, but so long as the client
    * has sufficient permissions to do so (it doesn't have HACK access yet) it
    * is safe enough.  Note that if you load a savegame the topology will be
-   * set but then overwritten during the load. */
-  my_snprintf(buf, sizeof(buf), "/set topology %d",
-	      (TF_WRAPX
-	       | ((tileset_is_isometric(tileset)
-		   && tileset_hex_height(tileset) == 0) ? TF_ISO : 0)
-	       | ((tileset_hex_width(tileset) != 0
-		   || tileset_hex_height(tileset) != 0) ? TF_HEX : 0)));
-  send_chat(buf);
+   * set but then overwritten during the load.
+   *
+   * Don't send it now, it will be sent to the server when receiving the
+   * server setting infos. */
+  {
+    char buf[16];
+
+    my_snprintf(buf, sizeof(buf), "%d",
+                (TF_WRAPX
+                 | ((tileset_is_isometric(tileset)
+                    && tileset_hex_height(tileset) == 0) ? TF_ISO : 0)
+                 | ((tileset_hex_width(tileset) != 0
+                    || tileset_hex_height(tileset) != 0) ? TF_HEX : 0)));
+    desired_settable_option_update("topology", buf, FALSE);
+  }
 
   return TRUE;
-#endif
+#endif /* HAVE_WORKING_FORK || WIN32_NATIVE */
+}
+
+/*************************************************************************
+  generate a random string.
+*************************************************************************/
+static void randomize_string(char *str, size_t n)
+{
+  const char chars[] =
+    "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+  int i;
+
+  for (i = 0; i < n - 1; i++) {
+    str[i] = chars[myrand(sizeof(chars) - 1)];
+  }
+  str[i] = '\0';
 }
 
 /*************************************************************************
@@ -437,29 +466,29 @@ void send_client_wants_hack(const char *filename)
     sz_strlcat(challenge_fullname, filename);
 
     /* generate an authentication token */ 
-    randomize_base64url_string(req.token, sizeof(req.token));
+    randomize_string(req.token, sizeof(req.token));
 
     section_file_init(&file);
     secfile_insert_str(&file, req.token, "challenge.token");
-    if (!section_file_save(&file, challenge_fullname, 0)) {
+    if (!section_file_save(&file, challenge_fullname, 0, FZ_PLAIN)) {
       freelog(LOG_ERROR, "Couldn't write token to temporary file: %s",
 	      challenge_fullname);
     }
     section_file_free(&file);
 
     /* tell the server what we put into the file */ 
-    send_packet_single_want_hack_req(&aconnection, &req);
+    send_packet_single_want_hack_req(&client.conn, &req);
   }
 }
 
 /**************************************************************** 
-  client has hack (or not).
+handle response (by the server) if the client has got hack or not.
 *****************************************************************/ 
 void handle_single_want_hack_reply(bool you_have_hack)
 {
   /* remove challenge file */
   if (challenge_fullname[0] != '\0') {
-    if (remove(challenge_fullname) == -1) {
+    if (fc_remove(challenge_fullname) == -1) {
       freelog(LOG_ERROR, "Couldn't remove temporary file: %s",
 	      challenge_fullname);
     }
@@ -467,46 +496,30 @@ void handle_single_want_hack_reply(bool you_have_hack)
   }
 
   if (you_have_hack) {
-    append_output_window(_("Established control over the server. "
+    output_window_append(ftc_client,
+                         _("Established control over the server. "
                            "You have command access level 'hack'."));
     client_has_hack = TRUE;
   } else if (is_server_running()) {
     /* only output this if we started the server and we NEED hack */
-    append_output_window(_("Failed to obtain the required access "
+    output_window_append(ftc_client,
+                         _("Failed to obtain the required access "
                            "level to take control of the server. "
                            "The server will now be shutdown."));
     client_kill_server(TRUE);
   }
 }
 
-/**************************************************************** 
-send server commands to start a saved game.
-*****************************************************************/ 
-void send_start_saved_game(void)
-{   
-  char buf[MAX_LEN_MSG];
-
-  send_chat("/set timeout 0");
-  my_snprintf(buf, sizeof(buf), "/take \"%s\" \"%s\"",
-      	      user_name, leader_name);
-  send_chat(buf);
-  send_chat("/start");
-}
-
 /**************************************************************** 
 send server command to save game.
 *****************************************************************/ 
 void send_save_game(char *filename)
 {   
-  char message[MAX_LEN_MSG];
-
   if (filename) {
-    my_snprintf(message, MAX_LEN_MSG, "/save %s", filename);
+    send_chat_printf("/save %s", filename);
   } else {
-    my_snprintf(message, MAX_LEN_MSG, "/save");
+    send_chat("/save");
   }
-
-  send_chat(message);
 }
 
 /**************************************************************************
