@@ -22,15 +22,16 @@
 
 #include <gtk/gtk.h>
 
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "packets.h"
 #include "support.h"
 #include "version.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "chatline.h"
-#include "clinet.h"
 #include "colors.h"
 #include "connectdlg_common.h"
 #include "dialogs.h"
@@ -77,10 +78,7 @@ static void filesel_response_callback(GtkWidget *w, gint id, gpointer data)
     if (is_save) {
       send_save_game(filename);
     } else {
-      char message[MAX_LEN_MSG];
-
-      my_snprintf(message, sizeof(message), "/load %s", filename);
-      send_chat(message);
+      send_chat_printf("/load %s", filename);
     }
 
     g_free(filename);
