@@ -22,22 +22,30 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+/* utility */
 #include "fcintl.h"
+#include "log.h"
+#include "support.h"
+
+/* common */
 #include "game.h"
 #include "map.h"
 #include "packets.h"
 #include "player.h"
-#include "support.h"
 #include "unit.h"
+#include "unitlist.h"
 
-#include "clinet.h"
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "control.h"
+#include "goto.h"
+#include "options.h"
+
+/* gui-gtk-2.0 */
 #include "dialogs.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
-#include "options.h"
 
 #include "gotodlg.h"
 
@@ -80,11 +88,9 @@ static void goto_cmd_callback(GtkWidget *dlg, gint arg)
       struct city *pdestcity = get_selected_city();
 
       if (pdestcity) {
-        struct unit *punit = get_unit_in_focus();
-
-        if (punit) {
+	unit_list_iterate(get_units_in_focus(), punit) {
           request_unit_airlift(punit, pdestcity);
-        }
+        } unit_list_iterate_end;
       }
     }
     break;
@@ -94,11 +100,9 @@ static void goto_cmd_callback(GtkWidget *dlg, gint arg)
       struct city *pdestcity = get_selected_city();
 
       if (pdestcity) {
-        struct unit *punit = get_unit_in_focus();
-
-        if (punit) {
-          send_goto_unit(punit, pdestcity->tile);
-        }
+	unit_list_iterate(get_units_in_focus(), punit) {
+          send_goto_tile(punit, pdestcity->tile);
+        } unit_list_iterate_end;
       }
     }
     break;
@@ -127,7 +131,7 @@ static void create_goto_dialog(void)
     GTK_RESPONSE_CANCEL,
     _("Air_lift"),
     CMD_AIRLIFT,
-    ("_Goto"),
+    _("_Goto"),
     CMD_GOTO,
     NULL);
   setup_dialog(dshell, toplevel);
@@ -145,7 +149,7 @@ static void create_goto_dialog(void)
   vbox = gtk_vbox_new(FALSE, 6);
   gtk_container_add(GTK_CONTAINER(label), vbox);
 
-  store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_BOOLEAN);
+  store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_INT);
   gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store),
     0, GTK_SORT_ASCENDING);
 
@@ -177,12 +181,12 @@ static void create_goto_dialog(void)
     "xalign", 0.0,
     "yalign", 0.5,
     NULL);
-  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
   gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
 
   all_toggle = gtk_check_button_new_with_mnemonic(_("Show _All Cities"));
-  gtk_box_pack_start(GTK_BOX(vbox), all_toggle, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), all_toggle, FALSE, FALSE, 0);
 
   g_signal_connect(all_toggle, "toggled", G_CALLBACK(update_goto_dialog), NULL);
 
@@ -204,7 +208,7 @@ popup the dialog
 *****************************************************************/
 void popup_goto_dialog(void)
 {
-  if (!can_client_issue_orders() || !get_unit_in_focus()) {
+  if (!can_client_issue_orders() || get_num_units_in_focus() == 0) {
     return;
   }
 
@@ -222,15 +226,15 @@ static struct city *get_selected_city(void)
 {
   GtkTreeModel *model;
   GtkTreeIter it;
-  char *name;
+  int city_id;
 
   if (!gtk_tree_selection_get_selected(selection, NULL, &it))
     return NULL;
 
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
 
-  gtk_tree_model_get(model, &it, 0, &name, -1);
-  return game_find_city_by_name(name);
+  gtk_tree_model_get(model, &it, 2, &city_id, -1);
+  return game_find_city_by_number(city_id);
 }
 
 /**************************************************************************
@@ -238,7 +242,6 @@ static struct city *get_selected_city(void)
 **************************************************************************/
 static void update_goto_dialog(GtkToggleButton *button)
 {
-  int i, j;
   GtkTreeIter it;
   gboolean all_cities;
 
@@ -246,18 +249,22 @@ static void update_goto_dialog(GtkToggleButton *button)
 
   gtk_list_store_clear(store);
 
-  for(i = 0, j = 0; i < game.nplayers; i++) {
-    if (!all_cities && i != game.player_idx)
+  players_iterate(pplayer) {
+    if (!all_cities && pplayer != client.conn.playing) {
       continue;
+    }
 
-    city_list_iterate(game.players[i].cities, pcity) {
+    city_list_iterate(pplayer->cities, pcity) {
       gtk_list_store_append(store, &it);
 
       /* FIXME: should use unit_can_airlift_to(). */
-      gtk_list_store_set(store, &it, 0, pcity->name, 1, pcity->airlift, -1);
-    }
-    city_list_iterate_end;
-  }
+      gtk_list_store_set(store, &it,
+                         0, city_name(pcity),
+                         1, pcity->airlift,
+                         2, pcity->id,
+                         -1);
+    } city_list_iterate_end;
+  } players_iterate_end;
 }
 
 /**************************************************************************
@@ -268,9 +275,17 @@ static void goto_selection_callback(GtkTreeSelection *selection, gpointer data)
   struct city *pdestcity;
 
   if((pdestcity = get_selected_city())) {
-    struct unit *punit = get_unit_in_focus();
+    bool can_airlift = FALSE;
+
+    unit_list_iterate(get_units_in_focus(), punit) {
+      if (unit_can_airlift_to(punit, pdestcity)) {
+	can_airlift = TRUE;
+	break;
+      }
+    } unit_list_iterate_end;
+
     center_tile_mapcanvas(pdestcity->tile);
-    if(punit && unit_can_airlift_to(punit, pdestcity)) {
+    if (can_airlift) {
       gtk_dialog_set_response_sensitive(GTK_DIALOG(dshell), CMD_AIRLIFT, TRUE);
       return;
     }
