@@ -23,13 +23,17 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+/* utility */
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 #include "support.h"
 
-#include "colors.h"
+/* client */
 #include "options.h"
+
+/* gui-gtk-2.0 */
+#include "colors.h"
 #include "gui_main.h"
 
 #include "gui_stuff.h"
@@ -245,9 +249,13 @@ gint gtk_tree_selection_get_row(GtkTreeSelection *selection)
 **************************************************************************/
 void gtk_tree_view_focus(GtkTreeView *view)
 {
+  GtkTreeModel *model;
   GtkTreePath *path;
+  GtkTreeIter iter;
 
-  if ((path = gtk_tree_path_new_first())) {
+  if ((model = gtk_tree_view_get_model(view))
+      && gtk_tree_model_get_iter_first(model, &iter)
+      && (path = gtk_tree_model_get_path(model, &iter))) {
     gtk_tree_view_set_cursor(view, path, NULL, FALSE);
     gtk_tree_path_free(path);
     gtk_widget_grab_focus(GTK_WIDGET(view));
@@ -269,7 +277,7 @@ static void close_callback(GtkDialog *dialog, gpointer data)
 ***********************************************************************/
 void setup_dialog(GtkWidget *shell, GtkWidget *parent)
 {
-  if (dialogs_on_top || fullscreen_mode) {
+  if (gui_gtk2_dialogs_on_top || fullscreen_mode) {
     gtk_window_set_transient_for(GTK_WINDOW(shell),
                                  GTK_WINDOW(parent));
     gtk_window_set_type_hint(GTK_WINDOW(shell),
@@ -487,7 +495,7 @@ static gboolean click_on_tab_callback(GtkWidget* w,
 
 /**************************************************************************
   Creates a new dialog. It will be a tab or a window depending on the
-  current user setting of 'enable_tabs'.
+  current user setting of 'gui_gtk2_enable_tabs'.
   Sets pdlg to point to the dialog once it is create, Zeroes pdlg on
   dialog destruction.
   user_data will be passed through response function
@@ -510,7 +518,7 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
   dlg->default_width = 200;
   dlg->default_height = 300;
 
-  if (enable_tabs) {
+  if (gui_gtk2_enable_tabs) {
     dlg->type = GUI_DIALOG_TAB;
   } else {
     dlg->type = GUI_DIALOG_WINDOW;
@@ -521,7 +529,7 @@ void gui_dialog_new(struct gui_dialog **pdlg, GtkNotebook *notebook,
   }
   dlg->gui_button = gtk_size_group_new(GTK_SIZE_GROUP_BOTH);
 
-  if (enable_tabs && notebook == GTK_NOTEBOOK(bottom_notebook)) {
+  if (gui_gtk2_enable_tabs && notebook == GTK_NOTEBOOK(bottom_notebook)) {
     vbox = gtk_hbox_new(FALSE, 0);
     action_area = gtk_vbox_new(FALSE, 2);
   } else {
@@ -975,6 +983,88 @@ void gui_dialog_set_return_dialog(struct gui_dialog *dlg,
   }
 }
 
+/**************************************************************************
+  Updates a gui font style.
+**************************************************************************/
+void gui_update_font(const char *font_name, const char *font_value)
+{
+  char str[512];
+
+  my_snprintf(str, sizeof(str),
+              "style \"ext-%s\" {\n"
+              "  font_name = \"%s\"\n"
+              "}\n"
+              "\n"
+              "widget \"Freeciv*.%s\" style \"ext-%s\"",
+              font_name, font_value, font_name, font_name);
+
+  gtk_rc_parse_string(str);
+}
+
+/****************************************************************************
+  Update a font option which is not attached to a widget.
+****************************************************************************/
+void gui_update_font_full(const char *font_name, const char *font_value,
+                          GtkStyle **pstyle)
+{
+  GtkSettings *settings;
+  GdkScreen *screen;
+  GtkStyle *style;
+  char buf[64];
+
+  gui_update_font(font_name, font_value);
+
+  screen = gdk_screen_get_default();
+  settings = gtk_settings_get_for_screen(screen);
+
+  my_snprintf(buf, sizeof(buf), "Freeciv*.%s", font_name);
+  style = gtk_rc_get_style_by_paths(settings, buf, NULL, G_TYPE_NONE);
+
+  if (style) {
+    g_object_ref(style);
+  } else {
+    style = gtk_style_new();
+  }
+
+  if (*pstyle) {
+    g_object_unref(*pstyle);
+  }
+  *pstyle = style;
+}
+
+/****************************************************************************
+  Temporarily disable signal invocation of the given callback for the given
+  GObject. Re-enable the signal with enable_gobject_callback.
+****************************************************************************/
+void disable_gobject_callback(GObject *obj, GCallback cb)
+{
+  gulong hid;
+
+  if (!obj || !cb) {
+    return;
+  }
+
+  hid = g_signal_handler_find(obj, G_SIGNAL_MATCH_FUNC,
+                              0, 0, NULL, cb, NULL);
+  g_signal_handler_block(obj, hid);
+}
+
+/****************************************************************************
+  Re-enable a signal callback blocked by disable_gobject_callback.
+****************************************************************************/
+void enable_gobject_callback(GObject *obj, GCallback cb)
+{
+  gulong hid;
+
+  if (!obj || !cb) {
+    return;
+  }
+
+  hid = g_signal_handler_find(obj, G_SIGNAL_MATCH_FUNC,
+                              0, 0, NULL, cb, NULL);
+  g_signal_handler_unblock(obj, hid);
+}
+
 /**************************************************************************
   Convenience function to add a column to a GtkTreeView. Returns the added
   column, or NULL if an error occured.
