@@ -24,8 +24,9 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+/* common & utility */
 #include "fcintl.h"
-#include "game.h"
+#include "game.h" /* setting_class_is_changeable() */
 #include "government.h"
 #include "log.h"
 #include "packets.h"
@@ -33,11 +34,11 @@
 #include "support.h"
 #include "unitlist.h"
 
+/* client */
 #include "chatline_common.h"
 #include "cityrep.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "control.h"
 #include "dialogs.h"
 #include "gui_main.h"
@@ -79,11 +80,7 @@ static void economy_command_callback(struct gui_dialog *dlg, int response,
                                      gpointer data);
 static void economy_selection_callback(GtkTreeSelection *selection,
 				       gpointer data);
-struct economy_row {
-  int is_impr;
-  int type;
-};
-static struct economy_row economy_row_type[U_LAST + B_LAST];
+static struct universal economy_row_type[U_LAST + B_LAST];
 
 static struct gui_dialog *economy_dialog_shell = NULL;
 static GtkWidget *economy_label2;
@@ -148,8 +145,8 @@ void popup_science_dialog(bool raise)
   }
 
   if (can_client_issue_orders()
-      && get_player_research(game.player_ptr)->tech_goal == A_UNSET
-      && get_player_research(game.player_ptr)->researching == A_UNSET) {
+      && A_UNSET == get_player_research(client.conn.playing)->tech_goal
+      && A_UNSET == get_player_research(client.conn.playing)->researching) {
     gui_dialog_alert(science_dialog_shell);
   } else {
     gui_dialog_present(science_dialog_shell);
@@ -185,23 +182,26 @@ static void button_release_event_callback(GtkWidget *widget,
   if (tech == A_NONE) {
     return;
   }
-  if (event->button == 1 && can_client_issue_orders()) {
-    /* LMB: set research or research goal */
-    switch (get_invention(game.player_ptr, tech)) {
-    case TECH_REACHABLE:
-      dsend_packet_player_research(&aconnection, tech);
-      break;
-    case TECH_UNKNOWN:
-      dsend_packet_player_tech_goal(&aconnection, tech);
-      break;
-    case TECH_KNOWN:
-      break;
-    }
-  } else if (event->button == 3) {
+
+  if (event->button == 3) {
     /* RMB: get help */
     /* FIXME: this should work for ctrl+LMB or shift+LMB (?) too */
-    popup_help_dialog_typed(advance_name_for_player(game.player_ptr, tech), HELP_TECH);
-  }
+    popup_help_dialog_typed(advance_name_for_player(client.conn.playing, tech), HELP_TECH);
+  } else {
+    if (event->button == 1 && can_client_issue_orders()) {
+      /* LMB: set research or research goal */
+      switch (player_invention_state(client.conn.playing, tech)) {
+       case TECH_PREREQS_KNOWN:
+         dsend_packet_player_research(&client.conn, tech);
+         break;
+       case TECH_UNKNOWN:
+         dsend_packet_player_tech_goal(&client.conn, tech);
+         break;
+       case TECH_KNOWN:
+         break;
+      }
+    }
+  } 
 }
 
 /****************************************************************************
@@ -226,12 +226,20 @@ static void update_science_drawing_area(GtkWidget *widget, gpointer data)
 static GtkWidget *create_reqtree_diagram(void)
 {
   GtkWidget *sw;
-  struct reqtree *reqtree = create_reqtree(game.player_ptr);
+  struct reqtree *reqtree;
   GtkAdjustment* adjustment;
   int width, height;
   int x;
   Tech_type_id researching;
 
+  if (can_conn_edit(&client.conn)) {
+    /* Show all techs in editor mode, not only currently reachable ones */
+    reqtree = create_reqtree(NULL);
+  } else {
+    /* Show only at some point reachable techs */
+    reqtree = create_reqtree(client.conn.playing);
+  }
+
   get_reqtree_dimensions(reqtree, &width, &height);
 
   sw = gtk_scrolled_window_new(NULL, NULL);
@@ -243,9 +251,9 @@ static GtkWidget *create_reqtree_diagram(void)
   g_signal_connect(G_OBJECT(science_drawing_area), "button-release-event",
                    G_CALLBACK(button_release_event_callback), NULL);
   gtk_widget_add_events(science_drawing_area,
-                        GDK_BUTTON_RELEASE_MASK | GDK_BUTTON2_MOTION_MASK | 
-			GDK_BUTTON3_MOTION_MASK);
-			 
+                        GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
+                        | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK);
+
   gtk_layout_set_size(GTK_LAYOUT(science_drawing_area), width, height);
 
   gtk_container_add(GTK_CONTAINER(sw), science_drawing_area);
@@ -256,8 +264,8 @@ static GtkWidget *create_reqtree_diagram(void)
   adjustment = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(sw));
   
   /* Center on currently researched node */
-  if (game.player_ptr) {
-    researching = get_player_research(game.player_ptr)->researching;
+  if (NULL != client.conn.playing) {
+    researching = get_player_research(client.conn.playing)->researching;
   } else {
     researching = A_UNSET;
   }
@@ -343,6 +351,35 @@ void create_science_dialog(bool make_modal)
   gtk_widget_grab_focus(science_change_menu_button);
 }
 
+/****************************************************************************
+  Resize and redraw the requirement tree.
+****************************************************************************/
+void science_dialog_redraw(void)
+{
+  struct reqtree *reqtree;
+  int width, height;
+
+  if (NULL == science_dialog_shell) {
+    /* Not existant. */
+    return;
+  }
+
+  if (can_conn_edit(&client.conn)) {
+    /* Show all techs in editor mode, not only currently reachable ones */
+    reqtree = create_reqtree(NULL);
+  } else {
+    /* Show only at some point reachable techs */
+    reqtree = create_reqtree(client_player());
+  }
+
+  get_reqtree_dimensions(reqtree, &width, &height);
+  gtk_layout_set_size(GTK_LAYOUT(science_drawing_area), width, height);
+  g_object_set_data_full(G_OBJECT(science_drawing_area), "reqtree", reqtree,
+                         (GDestroyNotify) destroy_reqtree);
+
+  gtk_widget_queue_draw(science_drawing_area);
+}
+
 /****************************************************************************
   Called to set several texts in the science dialog.
 ****************************************************************************/
@@ -366,7 +403,7 @@ void science_change_callback(GtkWidget *widget, gpointer data)
   size_t to = (size_t) data;
 
   if (GTK_TOGGLE_BUTTON(science_help_toggle)->active) {
-    popup_help_dialog_typed(advance_name_for_player(game.player_ptr, to), HELP_TECH);
+    popup_help_dialog_typed(advance_name_for_player(client.conn.playing, to), HELP_TECH);
     /* Following is to make the menu go back to the current research;
      * there may be a better way to do this?  --dwp */
     science_dialog_update();
@@ -376,7 +413,7 @@ void science_change_callback(GtkWidget *widget, gpointer data)
 			     can_client_issue_orders());
     update_science_text();
     
-    dsend_packet_player_research(&aconnection, to);
+    dsend_packet_player_research(&client.conn, to);
   }
 }
 
@@ -388,7 +425,7 @@ void science_goal_callback(GtkWidget *widget, gpointer data)
   size_t to = (size_t) data;
 
   if (GTK_TOGGLE_BUTTON(science_help_toggle)->active) {
-    popup_help_dialog_typed(advance_name_for_player(game.player_ptr, to), HELP_TECH);
+    popup_help_dialog_typed(advance_name_for_player(client.conn.playing, to), HELP_TECH);
     /* Following is to make the menu go back to the current goal;
      * there may be a better way to do this?  --dwp */
     science_dialog_update();
@@ -396,7 +433,7 @@ void science_goal_callback(GtkWidget *widget, gpointer data)
   else {  
     gtk_label_set_text(GTK_LABEL(science_goal_label),
 		       get_science_goal_text(to));
-    dsend_packet_player_tech_goal(&aconnection, to);
+    dsend_packet_player_tech_goal(&client.conn, to);
   }
 }
 
@@ -408,10 +445,10 @@ static gint cmp_func(gconstpointer a_p, gconstpointer b_p)
   const gchar *a_str, *b_str;
   gint a = GPOINTER_TO_INT(a_p), b = GPOINTER_TO_INT(b_p);
 
-  a_str = advance_name_for_player(game.player_ptr, a);
-  b_str = advance_name_for_player(game.player_ptr, b);
+  a_str = advance_name_for_player(client.conn.playing, a);
+  b_str = advance_name_for_player(client.conn.playing, b);
 
-  return strcmp(a_str,b_str);
+  return compare_strings(a_str, b_str);
 }
 
 /****************************************************************
@@ -425,9 +462,9 @@ void science_dialog_update(void)
   GtkWidget *item;
   GList *sorting_list = NULL, *it;
   GtkSizeGroup *group1, *group2;
-  struct player_research *research = get_player_research(game.player_ptr);
+  struct player_research *research = get_player_research(client.conn.playing);
 
-  if (!game.player_ptr || !research || is_report_dialogs_frozen()) {
+  if (!research || is_report_dialogs_frozen()) {
     return;
   }
 
@@ -436,11 +473,11 @@ void science_dialog_update(void)
   gtk_label_set_text(GTK_LABEL(science_label), science_dialog_text());
   
   /* collect all researched techs in sorting_list */
-  for(i=A_FIRST; i<game.control.num_tech_types; i++) {
-    if ((get_invention(game.player_ptr, i)==TECH_KNOWN)) {
+  advance_index_iterate(A_FIRST, i) {
+    if (TECH_KNOWN == player_invention_state(client.conn.playing, i)) {
       sorting_list = g_list_prepend(sorting_list, GINT_TO_POINTER(i));
     }
-  }
+  } advance_index_iterate_end;
 
   /* sort them, and install them in the list */
   sorting_list = g_list_sort(sorting_list, cmp_func);
@@ -460,7 +497,7 @@ void science_dialog_update(void)
   gtk_widget_queue_resize(science_current_label);
  
   if (research->researching == A_UNSET) {
-    item = gtk_menu_item_new_with_label(advance_name_for_player(game.player_ptr,
+    item = gtk_menu_item_new_with_label(advance_name_for_player(client.conn.playing,
 						      A_NONE));
     gtk_menu_shell_append(GTK_MENU_SHELL(popupmenu), item);
   }
@@ -470,17 +507,18 @@ void science_dialog_update(void)
    */
   hist=0;
   if (!is_future_tech(research->researching)) {
-    for(i=A_FIRST; i<game.control.num_tech_types; i++) {
-      if(get_invention(game.player_ptr, i)!=TECH_REACHABLE)
+    advance_index_iterate(A_FIRST, i) {
+      if (TECH_PREREQS_KNOWN !=
+            player_invention_state(client.conn.playing, i)) {
 	continue;
+      }
 
       if (i == research->researching)
 	hist=i;
       sorting_list = g_list_prepend(sorting_list, GINT_TO_POINTER(i));
-    }
+    } advance_index_iterate_end;
   } else {
-    int value = (game.control.num_tech_types
-		 + research->future_tech + 1);
+    int value = (advance_count() + research->future_tech + 1);
 
     sorting_list = g_list_prepend(sorting_list, GINT_TO_POINTER(value));
   }
@@ -490,14 +528,13 @@ void science_dialog_update(void)
   for (i = 0; i < g_list_length(sorting_list); i++) {
     const gchar *data;
 
-    if (GPOINTER_TO_INT(g_list_nth_data(sorting_list, i)) <
-	game.control.num_tech_types) {
-      data = advance_name_for_player(game.player_ptr,
+    if (GPOINTER_TO_INT(g_list_nth_data(sorting_list, i)) < advance_count()) {
+      data = advance_name_for_player(client.conn.playing,
 			GPOINTER_TO_INT(g_list_nth_data(sorting_list, i)));
     } else {
       my_snprintf(text, sizeof(text), _("Future Tech. %d"),
 		  GPOINTER_TO_INT(g_list_nth_data(sorting_list, i))
-		  - game.control.num_tech_types);
+		  - advance_count());
       data=text;
     }
 
@@ -527,7 +564,7 @@ void science_dialog_update(void)
 		  research->tech_goal));
 
   if (research->tech_goal == A_UNSET) {
-    item = gtk_menu_item_new_with_label(advance_name_for_player(game.player_ptr,
+    item = gtk_menu_item_new_with_label(advance_name_for_player(client.conn.playing,
 						      A_NONE));
     gtk_menu_shell_append(GTK_MENU_SHELL(goalmenu), item);
   }
@@ -536,18 +573,17 @@ void science_dialog_update(void)
    * hist will hold afterwards the techid of the current choice
    */
   hist=0;
-  for(i=A_FIRST; i<game.control.num_tech_types; i++) {
-    if (tech_is_available(game.player_ptr, i)
-        && get_invention(game.player_ptr, i) != TECH_KNOWN
-        && advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST
-        && (num_unknown_techs_for_goal(game.player_ptr, i) < 11
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_reachable(client.conn.playing, i)
+        && TECH_KNOWN != player_invention_state(client.conn.playing, i)
+        && (11 > num_unknown_techs_for_goal(client.conn.playing, i)
 	    || i == research->tech_goal)) {
       if (i == research->tech_goal) {
 	hist = i;
       }
       sorting_list = g_list_prepend(sorting_list, GINT_TO_POINTER(i));
     }
-  }
+  } advance_index_iterate_end;
 
   group1 = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
   group2 = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
@@ -563,13 +599,13 @@ void science_dialog_update(void)
     hbox = gtk_hbox_new(FALSE, 18);
     gtk_container_add(GTK_CONTAINER(item), hbox);
 
-    label = gtk_label_new(advance_name_for_player(game.player_ptr, tech));
+    label = gtk_label_new(advance_name_for_player(client.conn.playing, tech));
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
     gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
     gtk_size_group_add_widget(group1, label);
 
     my_snprintf(text, sizeof(text), "%d",
-	num_unknown_techs_for_goal(game.player_ptr, tech));
+	num_unknown_techs_for_goal(client.conn.playing, tech));
 
     label = gtk_label_new(text);
     gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -631,10 +667,12 @@ void popdown_economy_report_dialog(void)
 void create_economy_report_dialog(bool make_modal)
 {
   const char *titles[5] = {
+    /* TRANS: Image header */
     _("Type"),
     Q_("?Building:Name"),
     _("Count"),
     _("Cost"),
+    /* TRANS: Upkeep total, count*cost */
     _("U Total")
   };
   int i;
@@ -668,7 +706,7 @@ void create_economy_report_dialog(bool make_modal)
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(economy_store));
   g_object_unref(economy_store);
-  gtk_widget_set_name(view, "small font");
+  gtk_widget_set_name(view, "small_font");
   gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
   economy_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
   g_signal_connect(economy_selection, "changed",
@@ -744,22 +782,30 @@ static void economy_selection_callback(GtkTreeSelection *selection,
 				       gpointer data)
 {
   gint row = gtk_tree_selection_get_row(selection);
-  int i = economy_row_type[row].type;
 
   if (row >= 0) {
-    if (economy_row_type[row].is_impr == TRUE) {
+    switch (economy_row_type[row].kind) {
+    case VUT_IMPROVEMENT:
+    {
       /* The user has selected an improvement type. */
-      bool is_sellable = (improvement_exists(i) && can_sell_building(i));
+      struct impr_type *pimprove = economy_row_type[row].value.building;
+      bool is_sellable = can_sell_building(pimprove);
 
       gtk_widget_set_sensitive(sellobsolete_command, is_sellable
 			       && can_client_issue_orders()
-			       && improvement_obsolete(game.player_ptr, i));
+			       && improvement_obsolete(client.conn.playing, pimprove));
       gtk_widget_set_sensitive(sellall_command, is_sellable
 			       && can_client_issue_orders());
-    } else {
+      break;
+    }
+    case VUT_UTYPE:
       /* An unit has been selected */
       gtk_widget_set_sensitive(sellall_command, can_client_issue_orders());
-    }
+      break;
+    default:
+      assert(0);
+      break;
+    };
   } else {
     /* No selection has been made. */
     gtk_widget_set_sensitive(sellobsolete_command, FALSE);
@@ -773,7 +819,6 @@ static void economy_selection_callback(GtkTreeSelection *selection,
 static void economy_command_callback(struct gui_dialog *dlg, int response,
                                      gpointer callback)
 {
-  int i, is_impr;
   gint row;
   GtkWidget *shell;
   char buf[1024];
@@ -785,17 +830,18 @@ static void economy_command_callback(struct gui_dialog *dlg, int response,
 
   /* sell obsolete and sell all. */
   row = gtk_tree_selection_get_row(economy_selection);
-  is_impr = economy_row_type[row].is_impr;
-  i = economy_row_type[row].type;
 
-  if (is_impr == TRUE) {
+  switch (economy_row_type[row].kind) {
+  case VUT_IMPROVEMENT:
+  {
+    struct impr_type *pimprove = economy_row_type[row].value.building;
     if (response == ECONOMY_SELL_ALL) {
       shell = gtk_message_dialog_new(
 	  NULL,
 	  GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
 	  GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
 	  _("Do you really wish to sell your %s?\n"),
-	  improvement_name_translation(i));
+	  improvement_name_translation(pimprove));
       setup_dialog(shell, gui_dialog_get_toplevel(dlg));
       gtk_window_set_title(GTK_WINDOW(shell), _("Sell Improvements"));
 
@@ -807,13 +853,19 @@ static void economy_command_callback(struct gui_dialog *dlg, int response,
       }
     }
 
-    sell_all_improvements(i, response!= ECONOMY_SELL_ALL, buf, sizeof(buf));
-  } else {
+    sell_all_improvements(pimprove, response!= ECONOMY_SELL_ALL, buf, sizeof(buf));
+    break;
+  }
+  case VUT_UTYPE:
     if (response== ECONOMY_SELL_OBSOLETE) {
       return;
     }
-    disband_all_units(utype_by_number(i), FALSE, buf, sizeof(buf));
-  }
+    disband_all_units(economy_row_type[row].value.utype, FALSE, buf, sizeof(buf));
+    break;
+  default:
+    assert(0);
+    break;
+  };
 
   shell = gtk_message_dialog_new(
       NULL,
@@ -859,8 +911,8 @@ void economy_report_dialog_update(void)
       gtk_list_store_set_value(economy_store, &it, 1, &value);
       g_value_unset(&value);
 
-      economy_row_type[i].is_impr = TRUE;
-      economy_row_type[i].type = p->type;
+      economy_row_type[i].kind = VUT_IMPROVEMENT;
+      economy_row_type[i].value.building = p->type;
     }
 
     nbr_impr = entries_used;
@@ -878,8 +930,8 @@ void economy_report_dialog_update(void)
       gtk_list_store_set_value(economy_store, &it, 1, &value);
       g_value_unset(&value);
     
-      economy_row_type[i + nbr_impr].is_impr = FALSE;
-      economy_row_type[i + nbr_impr].type = entries_units[i].type->index;
+      economy_row_type[i + nbr_impr].kind = VUT_UTYPE;
+      economy_row_type[i + nbr_impr].value.utype = entries_units[i].type;
     }
 
     my_snprintf(economy_total, sizeof(economy_total),
@@ -974,7 +1026,7 @@ void create_activeunits_report_dialog(bool make_modal)
 
   view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(activeunits_store));
   g_object_unref(activeunits_store);
-  gtk_widget_set_name(view, "small font");
+  gtk_widget_set_name(view, "small_font");
   gtk_tree_view_columns_autosize(GTK_TREE_VIEW(view));
   activeunits_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
   g_signal_connect(activeunits_selection, "changed",
@@ -1065,7 +1117,7 @@ static void activeunits_selection_callback(GtkTreeSelection *selection,
     gui_dialog_set_response_sensitive(activeunits_dialog_shell,
 		ACTIVEUNITS_UPGRADE,
 		(can_client_issue_orders()
-		 && can_upgrade_unittype(game.player_ptr, utype) != NULL));
+		 && NULL != can_upgrade_unittype(client.conn.playing, utype)));
   }
 }
 
@@ -1079,10 +1131,11 @@ static struct unit *find_nearest_unit(const struct unit_type *type,
   int best_dist = FC_INFINITY;
 
   best_candidate = NULL;
-  if (!game.player_ptr) {
+  if (NULL == client.conn.playing) {
     return NULL;
   }
-  unit_list_iterate(game.player_ptr->units, punit) {
+
+  unit_list_iterate(client.conn.playing->units, punit) {
     if (unit_type(punit) == type) {
       if (punit->focus_status==FOCUS_AVAIL
 	  && punit->moves_left > 0
@@ -1147,7 +1200,7 @@ static void activeunits_command_callback(struct gui_dialog *dlg, int response,
     }
   } else if (can_client_issue_orders()) {
     GtkWidget *shell;
-    struct unit_type *ut2 = can_upgrade_unittype(game.player_ptr, utype1);
+    struct unit_type *ut2 = can_upgrade_unittype(client.conn.playing, utype1);
 
     shell = gtk_message_dialog_new(
 	  NULL,
@@ -1157,14 +1210,14 @@ static void activeunits_command_callback(struct gui_dialog *dlg, int response,
 	    "Treasury contains %d gold."),
 	  utype_name_translation(utype1),
 	  utype_name_translation(ut2),
-	  unit_upgrade_price(game.player_ptr, utype1, ut2),
-	  game.player_ptr->economic.gold);
+	  unit_upgrade_price(client.conn.playing, utype1, ut2),
+	  client.conn.playing->economic.gold);
     setup_dialog(shell, gui_dialog_get_toplevel(dlg));
 
     gtk_window_set_title(GTK_WINDOW(shell), _("Upgrade Obsolete Units"));
 
     if (gtk_dialog_run(GTK_DIALOG(shell)) == GTK_RESPONSE_YES) {
-      dsend_packet_unit_type_upgrade(&aconnection, utype1->index);
+      dsend_packet_unit_type_upgrade(&client.conn, utype_number(utype1));
     }
 
     gtk_widget_destroy(shell);
@@ -1178,7 +1231,7 @@ void activeunits_report_dialog_update(void)
 {
   struct repoinfo {
     int active_count;
-    int upkeep[O_MAX];
+    int upkeep[O_LAST];
     int building_count;
   };
 
@@ -1187,7 +1240,7 @@ void activeunits_report_dialog_update(void)
   }
 
   gtk_list_store_clear(activeunits_store);
-  if (game.player_ptr) {
+  if (client_has_player()) {
     int    k, can;
     struct repoinfo unitarray[U_LAST];
     struct repoinfo unittotals;
@@ -1197,28 +1250,29 @@ void activeunits_report_dialog_update(void)
     gtk_list_store_clear(activeunits_store);
 
     memset(unitarray, '\0', sizeof(unitarray));
-    unit_list_iterate(game.player_ptr->units, punit) {
-      (unitarray[unit_type(punit)->index].active_count)++;
-      if (punit->homecity) {
-	output_type_iterate(o) {
-	  unitarray[unit_type(punit)->index].upkeep[o] += punit->upkeep[o];
-	} output_type_iterate_end;
+    unit_list_iterate(client_player()->units, punit) {
+      struct repoinfo *info = unitarray + utype_index(unit_type(punit));
+
+      if (0 != punit->homecity) {
+        output_type_iterate(o) {
+          info->upkeep[o] += punit->upkeep[o];
+        } output_type_iterate_end;
       }
+      info->active_count++;
     } unit_list_iterate_end;
-    city_list_iterate(game.player_ptr->cities,pcity) {
-      if (pcity->production.is_unit) {
-	(unitarray[pcity->production.value].building_count)++;
+    city_list_iterate(client_player()->cities, pcity) {
+      if (VUT_UTYPE == pcity->production.kind) {
+        unitarray[utype_index(pcity->production.value.utype)].building_count++;
       }
-    }
-    city_list_iterate_end;
+    } city_list_iterate_end;
 
     k = 0;
     memset(&unittotals, '\0', sizeof(unittotals));
     unit_type_iterate(punittype) {
-      Unit_type_id uti = punittype->index;
+      Unit_type_id uti = utype_index(punittype);
       if (unitarray[uti].active_count > 0
 	  || unitarray[uti].building_count > 0) {
-	can = (can_upgrade_unittype(game.player_ptr, punittype) != NULL);
+	can = (NULL != can_upgrade_unittype(client.conn.playing, punittype));
 	
         gtk_list_store_append(activeunits_store, &it);
 	gtk_list_store_set(activeunits_store, &it,
@@ -1283,14 +1337,18 @@ static void create_endgame_report(struct packet_endgame_report *packet)
     N_("Player\n"),
     N_("Score\n"),
     N_("Population\n"),
+    /* TRANS: "M goods" = million goods */
     N_("Trade\n(M goods)"), 
+    /* TRANS: "M tons" = million tons */
     N_("Production\n(M tons)"), 
     N_("Cities\n"),
     N_("Technologies\n"),
     N_("Military Service\n(months)"), 
     N_("Wonders\n"),
     N_("Research Speed\n(%)"), 
+    /* TRANS: "sq. mi." is abbreviation for "square miles" */
     N_("Land Area\n(sq. mi.)"), 
+    /* TRANS: "sq. mi." is abbreviation for "square miles" */
     N_("Settled Area\n(sq. mi.)"), 
     N_("Literacy\n(%)"), 
     N_("Spaceship\n")
@@ -1325,7 +1383,7 @@ static void create_endgame_report(struct packet_endgame_report *packet)
   scores_store = gtk_list_store_newv(ARRAY_SIZE(model_types), model_types);
   scores_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(scores_store));
   g_object_unref(scores_store);
-  gtk_widget_set_name(scores_list, "small font");
+  gtk_widget_set_name(scores_list, "small_font");
     
   for (i = 0; i < NUM_SCORE_COLS; i++) {
     GtkCellRenderer *renderer;
@@ -1395,9 +1453,8 @@ static void option_changed_callback(GtkWidget *widget, gpointer data)
 /*************************************************************************
   helper function for server options dialog
 *************************************************************************/
-static void settable_options_processing(GtkWidget *final)
+static void settable_options_processing(GtkWidget *final, gboolean local_update)
 {
-  char buffer[MAX_LEN_MSG];
   const char *desired_string;
   GtkWidget *w = final;
 
@@ -1405,38 +1462,33 @@ static void settable_options_processing(GtkWidget *final)
     struct options_settable *o =
       (struct options_settable *)g_object_get_data(G_OBJECT(w), "changed");
 
-    /* if the entry has been changed, then send the changes to the server */
+    /* If the entry has been changed, then send the changes to the server. */
     if (NULL != o) {
-      /* append the name of the option */
-      my_snprintf(buffer, sizeof(buffer), "/set %s ", gtk_widget_get_name(w));
+      desired_string = NULL;
 
-      /* append the setting */
+      /* Get the setting string value. */
       switch (o->stype) {
       case SSET_BOOL:
-        o->desired_val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
-        sz_strlcat(buffer, o->desired_val ? "1" : "0");
+        desired_string = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w))
+                          ? "1" : "0");
         break;
       case SSET_INT:
-        o->desired_val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
-        sz_strlcat(buffer, gtk_entry_get_text(GTK_ENTRY(w)));
-        break;
       case SSET_STRING:
         desired_string = gtk_entry_get_text(GTK_ENTRY(w));
-        if (NULL != desired_string) {
-          if (NULL != o->desired_strval) {
-            free(o->desired_strval);
-          }
-          o->desired_strval = mystrdup(desired_string);
-          sz_strlcat(buffer, desired_string);
-        }
-        break;
-      default:
-        freelog(LOG_ERROR,
-                "settable_options_processing() bad type %d.",
-                o->stype);
         break;
       };
-      send_chat(buffer);
+
+      if (NULL != desired_string) {
+        /* Send to server. */
+        send_chat_printf("/set %s %s", gtk_widget_get_name(w), desired_string);
+        if (local_update) {
+          /* Make a local change of the desired value for this option,
+           * probably because we want to save it before the server send its
+           * new value to us. */
+          desired_settable_option_update(gtk_widget_get_name(w),
+                                         desired_string, TRUE);
+        }
+      }
     }
 
     /* using the linked list, work backwards and check the previous widget */
@@ -1451,11 +1503,12 @@ static void settable_options_callback(GtkWidget *win, gint rid, GtkWidget *w)
 {
   switch (rid) {
   case GTK_RESPONSE_ACCEPT:
-    settable_options_processing(w);
-    save_options();
+    desired_settable_options_update();
+    settable_options_processing(w, TRUE);
+    options_save();
     break;
   case GTK_RESPONSE_APPLY:
-    settable_options_processing(w);
+    settable_options_processing(w, FALSE);
     break;
   default:
     break;
