@@ -32,20 +32,26 @@
 #include <X11/Xaw/AsciiText.h>  
 #include <X11/Xaw/Viewport.h>
 
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
-#include "map.h"
+#include "log.h"
 #include "mem.h"
-#include "packets.h"
-#include "player.h"
 #include "shared.h"
 #include "support.h"
 
-#include "chatline.h"
-#include "climisc.h"
-#include "clinet.h"
+/* common */
 #include "diptreaty.h"
+#include "government.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
+
+/* client */
+#include "client_main.h"
+#include "climisc.h"
+
+/* gui-xaw */
+#include "chatline.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
@@ -92,7 +98,7 @@ struct Diplomacy_dialog {
 
   Widget dip_erase_clause_command;
   
-  char clauselist_strings[MAX_NUM_CLAUSES+1][64];
+  char clauselist_strings[MAX_NUM_CLAUSES+1][128];
   char *clauselist_strings_ptrs[MAX_NUM_CLAUSES+1];
 };
 
@@ -107,7 +113,7 @@ static char *dummy_clause_list_strings[]
     TYPED_LIST_ITERATE(struct Diplomacy_dialog, dialoglist, pdialog)
 #define dialog_list_iterate_end  LIST_ITERATE_END
 
-static struct dialog_list dialog_list;
+static struct dialog_list *dialog_list = NULL;
 static bool dialog_list_list_has_been_initialised = FALSE;
 
 struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0, 
@@ -193,7 +199,7 @@ void handle_diplomacy_create_clause(int counterpart, int giver,
     return;
   }
 
-  add_clause(&pdialog->treaty, get_player(giver), type, value);
+  add_clause(&pdialog->treaty, player_by_number(giver), type, value);
   update_diplomacy_dialog(pdialog);
 }
 
@@ -209,7 +215,7 @@ void handle_diplomacy_remove_clause(int counterpart, int giver,
     return;
   }
 
-  remove_clause(&pdialog->treaty, get_player(giver), type, value);
+  remove_clause(&pdialog->treaty, player_by_number(giver), type, value);
   update_diplomacy_dialog(pdialog);
 }
 
@@ -220,7 +226,7 @@ static void popup_diplomacy_dialog(int other_player_id)
 {
   struct Diplomacy_dialog *pdialog = find_diplomacy_dialog(other_player_id);
 
-  if (game.player_ptr->ai.control) {
+  if (NULL == client.conn.playing || client.conn.playing->ai_data.control) {
     return;			/* Don't show if we are AI controlled. */
   }
 
@@ -228,8 +234,8 @@ static void popup_diplomacy_dialog(int other_player_id)
     Position x, y;
     Dimension width, height;
 
-    pdialog = create_diplomacy_dialog(game.player_ptr,
-				      get_player(other_player_id));
+    pdialog = create_diplomacy_dialog(client.conn.playing,
+				      player_by_number(other_player_id));
     XtVaGetValues(toplevel, XtNwidth, &width, XtNheight, &height, NULL);
     XtTranslateCoords(toplevel, (Position) width / 10,
 		      (Position) height / 10, &x, &y);
@@ -246,23 +252,25 @@ static void popup_diplomacy_dialog(int other_player_id)
 static int fill_diplomacy_tech_menu(Widget popupmenu, 
 				    struct player *plr0, struct player *plr1)
 {
-  int i, flag;
+  int flag = 0;
   
-  for(i=A_FIRST, flag=0; i<game.num_tech_types; i++) {
-    if (get_invention(plr0, i) == TECH_KNOWN
-        && (get_invention(plr1, i) == TECH_UNKNOWN
-	    || get_invention(plr1, i) == TECH_REACHABLE)
-        && tech_is_available(plr1, i)) {
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_state(plr0, i) == TECH_KNOWN
+        && player_invention_reachable(plr1, i)
+        && (player_invention_state(plr1, i) == TECH_UNKNOWN
+	    || player_invention_state(plr1, i) == TECH_PREREQS_KNOWN)) {
       Widget entry=
-	XtVaCreateManagedWidget(advances[i].name, smeBSBObjectClass, 
-				popupmenu, NULL);
+	XtVaCreateManagedWidget(advance_name_translation(advance_by_number(i)),
+				smeBSBObjectClass,
+				popupmenu,
+				NULL);
       XtAddCallback(entry, XtNcallback, diplomacy_dialog_tech_callback,
-			 INT_TO_XTPOINTER((plr0->player_no << 24) |
-					 (plr1->player_no << 16) |
+			 INT_TO_XTPOINTER((player_number(plr0) << 24) |
+					 (player_number(plr1) << 16) |
 					 i));
       flag=1;
     }
-  }
+  } advance_index_iterate_end;
   return flag;
 }
 
@@ -276,7 +284,7 @@ cities visible to requesting player.
 static int fill_diplomacy_city_menu(Widget popupmenu, 
 				    struct player *plr0, struct player *plr1)
 {
-  int i = 0, j = 0, n = city_list_size(&plr0->cities);
+  int i = 0, j = 0, n = city_list_size(plr0->cities);
   struct city **city_list_ptrs;
   if (n>0) {
     city_list_ptrs = fc_malloc(sizeof(struct city*)*n);
@@ -295,11 +303,11 @@ static int fill_diplomacy_city_menu(Widget popupmenu,
   
   for(j=0; j<i; j++) {
     Widget entry=
-      XtVaCreateManagedWidget(city_list_ptrs[j]->name, smeBSBObjectClass, 
+      XtVaCreateManagedWidget(city_name(city_list_ptrs[j]), smeBSBObjectClass, 
 			      popupmenu, NULL);
     XtAddCallback(entry, XtNcallback, diplomacy_dialog_city_callback,
-		  INT_TO_XTPOINTER((plr0->player_no << 24) |
-				   (plr1->player_no << 16) |
+		  INT_TO_XTPOINTER((player_number(plr0) << 24) |
+				   (player_number(plr1) << 16) |
 				   city_list_ptrs[j]->id));
   }
   free(city_list_ptrs);
@@ -320,7 +328,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   Widget entry;
 
   pdialog=fc_malloc(sizeof(struct Diplomacy_dialog));
-  dialog_list_insert(&dialog_list, pdialog);
+  dialog_list_prepend(dialog_list, pdialog);
   
   init_treaty(&pdialog->treaty, plr0, plr1);
   
@@ -354,7 +362,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 					       NULL);
   
   my_snprintf(buf, sizeof(buf), _("The %s offerings"),
-	      get_nation_name(plr0->nation));
+	      nation_adjective_for_player(plr0));
   pdialog->dip_headline0=XtVaCreateManagedWidget("dipheadline0", 
 						 labelWidgetClass, 
 						 pdialog->dip_form0, 
@@ -362,7 +370,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 						 NULL);   
 
   my_snprintf(buf, sizeof(buf), _("The %s offerings"),
-	      get_nation_name(plr1->nation));
+	      nation_adjective_for_player(plr1));
   pdialog->dip_headline1=XtVaCreateManagedWidget("dipheadline1", 
 						 labelWidgetClass, 
 						 pdialog->dip_form1, 
@@ -538,12 +546,12 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 	      _("This Eternal Treaty\n"
 		 "marks the results of the diplomatic work between\n"
 		 "The %s %s %s\nand\nThe %s %s %s"),
-	  get_nation_name(plr0->nation),
-	  get_ruler_title(plr0->government, plr0->is_male, plr0->nation),
-	  plr0->name,
-	  get_nation_name(plr1->nation),
-	  get_ruler_title(plr1->government, plr1->is_male, plr1->nation),
-	  plr1->name);
+	  nation_adjective_for_player(plr0),
+	  ruler_title_translation(plr0),
+	  player_name(plr0),
+	  nation_adjective_for_player(plr1),
+	  ruler_title_translation(plr1),
+	  player_name(plr1));
   
   pheadlinem=create_centered_string(buf);
   pdialog->dip_headline1=XtVaCreateManagedWidget("dipheadlinem", 
@@ -575,7 +583,8 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   XtVaSetValues(pdialog->dip_view, XtNwidth, width, NULL); 
   XtVaSetValues(pdialog->dip_clauselist, XtNwidth, width, NULL); 
 
-  my_snprintf(buf, sizeof(buf), _("%s view:"), get_nation_name(plr0->nation));
+  my_snprintf(buf, sizeof(buf), _("%s view:"),
+              nation_adjective_for_player(plr0));
   pdialog->dip_acceptlabel0=XtVaCreateManagedWidget("dipacceptlabel0",
 						    labelWidgetClass, 
 						    pdialog->dip_formm, 
@@ -586,7 +595,8 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 						    pdialog->dip_formm, 
 						    XtNbitmap, get_thumb_pixmap(0),
 						    NULL);
-  my_snprintf(buf, sizeof(buf), _("%s view:"), get_nation_name(plr1->nation));
+  my_snprintf(buf, sizeof(buf), _("%s view:"),
+              nation_adjective_for_player(plr1));
   pdialog->dip_acceptlabel1=XtVaCreateManagedWidget("dipacceptlabel1",
 						    labelWidgetClass, 
 						    pdialog->dip_formm, 
@@ -681,7 +691,7 @@ void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 }
 
 /****************************************************************
-...
+  Callback for trading techs
 *****************************************************************/
 void diplomacy_dialog_tech_callback(Widget w, XtPointer client_data,
 				    XtPointer call_data)
@@ -690,13 +700,13 @@ void diplomacy_dialog_tech_callback(Widget w, XtPointer client_data,
   int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
   int tech = choice & 0xffff;
 
-  if (giver == game.player_idx) {
+  if (player_by_number(giver) == client.conn.playing) {
     other = dest;
   } else {
     other = giver;
   }
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+  dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
 					   CLAUSE_ADVANCE, tech);
 }
 
@@ -711,13 +721,13 @@ void diplomacy_dialog_city_callback(Widget w, XtPointer client_data,
   int giver = (choice >> 24) & 0xff, dest = (choice >> 16) & 0xff, other;
   int city = choice & 0xffff;
 
-  if (giver == game.player_idx) {
+  if (player_by_number(giver) == client.conn.playing) {
     other = dest;
   } else {
     other = giver;
   }
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection, other, giver,
+  dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
 					   CLAUSE_CITY, city);
 }
 
@@ -735,10 +745,9 @@ void diplomacy_dialog_erase_clause_callback(Widget w, XtPointer client_data,
 
     clause_list_iterate(pdialog->treaty.clauses, pclause) {
       if (i == ret->list_index) {
-	dsend_packet_diplomacy_remove_clause_req(&aconnection,
-						 pdialog->treaty.plr1->
-						 player_no,
-						 pclause->from->player_no,
+	dsend_packet_diplomacy_remove_clause_req(&client.conn,
+						 player_number(pdialog->treaty.plr1),
+						 player_number(pclause->from),
 						 pclause->type,
 						 pclause->value);
 	return;
@@ -749,7 +758,7 @@ void diplomacy_dialog_erase_clause_callback(Widget w, XtPointer client_data,
 }
 
 /****************************************************************
-...
+  Callback for trading map
 *****************************************************************/
 void diplomacy_dialog_map_callback(Widget w, XtPointer client_data,
 				   XtPointer call_data)
@@ -760,13 +769,13 @@ void diplomacy_dialog_map_callback(Widget w, XtPointer client_data,
        pdialog->dip_map_menubutton0) ? pdialog->treaty.plr0 : pdialog->
       treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_MAP, 0);
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_MAP, 0);
 }
 
 /****************************************************************
-...
+  Callback for trading seamap
 *****************************************************************/
 void diplomacy_dialog_seamap_callback(Widget w, XtPointer client_data,
 				      XtPointer call_data)
@@ -777,9 +786,9 @@ void diplomacy_dialog_seamap_callback(Widget w, XtPointer client_data,
        pdialog->dip_map_menubutton0) ? pdialog->treaty.plr0 : pdialog->
       treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_SEAMAP,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_SEAMAP,
 					   0);
 }
 
@@ -793,9 +802,9 @@ void diplomacy_dialog_vision_callback(Widget w, XtPointer client_data,
   struct player *pgiver = (w == pdialog->dip_vision_button0) ?
       pdialog->treaty.plr0 : pdialog->treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, CLAUSE_VISION,
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), CLAUSE_VISION,
 					   0);
 }
 
@@ -809,9 +818,9 @@ static void diplomacy_dialog_add_pact_clause(Widget w, XtPointer client_data,
   struct player *pgiver = (w == pdialog->dip_vision_button0) ?
       pdialog->treaty.plr0 : pdialog->treaty.plr1;
 
-  dsend_packet_diplomacy_create_clause_req(&aconnection,
-					   pdialog->treaty.plr1->player_no,
-					   pgiver->player_no, type, 0);
+  dsend_packet_diplomacy_create_clause_req(&client.conn,
+					   player_number(pdialog->treaty.plr1),
+					   player_number(pgiver), type, 0);
 }
 
 /****************************************************************
@@ -856,8 +865,8 @@ void diplomacy_dialog_close_callback(Widget w, XtPointer client_data,
 {
   struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
 
-  dsend_packet_diplomacy_cancel_meeting_req(&aconnection,
-					    pdialog->treaty.plr1->player_no);
+  dsend_packet_diplomacy_cancel_meeting_req(&client.conn,
+					    player_number(pdialog->treaty.plr1));
   close_diplomacy_dialog(pdialog);
 }
 
@@ -870,8 +879,8 @@ void diplomacy_dialog_accept_callback(Widget w, XtPointer client_data,
 {
   struct Diplomacy_dialog *pdialog = (struct Diplomacy_dialog *) client_data;
 
-  dsend_packet_diplomacy_accept_treaty_req(&aconnection,
-					   pdialog->treaty.plr1->player_no);
+  dsend_packet_diplomacy_accept_treaty_req(&client.conn,
+					   player_number(pdialog->treaty.plr1));
 }
 
 
@@ -882,7 +891,7 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 {
   XtDestroyWidget(pdialog->dip_dialog_shell);
   
-  dialog_list_unlink(&dialog_list, pdialog);
+  dialog_list_unlink(dialog_list, pdialog);
   free(pdialog);
 }
 
@@ -891,10 +900,11 @@ void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
 *****************************************************************/
 static struct Diplomacy_dialog *find_diplomacy_dialog(int other_player_id)
 {
-  struct player *plr0 = game.player_ptr, *plr1 = get_player(other_player_id);
+  struct player *plr0 = client.conn.playing;
+  struct player *plr1 = player_by_number(other_player_id);
 
   if (!dialog_list_list_has_been_initialised) {
-    dialog_list_init(&dialog_list);
+    dialog_list = dialog_list_new();
     dialog_list_list_has_been_initialised = TRUE;
   }
 
@@ -938,14 +948,15 @@ void diplodlg_key_gold(Widget w)
     XtVaGetValues(w, XtNstring, &dp, NULL);
     if (sscanf(dp, "%d", &amount) == 1 && amount >= 0
 	&& amount <= pgiver->economic.gold) {
-      dsend_packet_diplomacy_create_clause_req(&aconnection,
-					       pdialog->treaty.plr1->
-					       player_no, pgiver->player_no,
+      dsend_packet_diplomacy_create_clause_req(&client.conn,
+					       player_number(pdialog->treaty.plr1),
+					       player_number(pgiver),
 					       CLAUSE_GOLD, amount);
       XtVaSetValues(w, XtNstring, "", NULL);
+    } else {
+      output_window_append(ftc_client,
+                           _("Invalid amount of gold specified."));
     }
-    else
-      append_output_window(_("Game: Invalid amount of gold specified."));
   }
 }
 
@@ -958,7 +969,7 @@ void close_all_diplomacy_dialogs(void)
     return;
   }
 
-  while (dialog_list_size(&dialog_list) > 0) {
-    close_diplomacy_dialog(dialog_list_get(&dialog_list, 0));
+  while (dialog_list_size(dialog_list) > 0) {
+    close_diplomacy_dialog(dialog_list_get(dialog_list, 0));
   }
 }
