@@ -18,6 +18,7 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <math.h> /* sqrt */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -29,9 +30,9 @@
 #include <X11/Xaw/AsciiText.h>
 #include <X11/Xaw/Tree.h>
 
+/* common & utility */
 #include "city.h"
 #include "fcintl.h"
-#include "game.h"
 #include "genlist.h"
 #include "government.h"
 #include "mem.h"
@@ -42,6 +43,8 @@
 #include "support.h"
 #include "version.h"
 
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "dialogs.h"
 #include "graphics.h"
@@ -251,7 +254,7 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
   char *bg="";
   char label[MAX_LEN_NAME+3];
   
-  type = (tech==A_LAST) ? TECH_UNKNOWN : get_invention(game.player_ptr, tech);
+  type = (tech==A_LAST) ? TECH_UNKNOWN : player_invention_state(client.conn.playing, tech);
   switch(type) {
     case TECH_UNKNOWN:
       bg=TREE_NODE_UNKNOWN_TECH_BG;
@@ -259,13 +262,14 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
     case TECH_KNOWN:
       bg=TREE_NODE_KNOWN_TECH_BG;
       break;
-    case TECH_REACHABLE:
+    case TECH_PREREQS_KNOWN:
       bg=TREE_NODE_REACHABLE_TECH_BG;
       break;
   }
   
   if(tech==A_LAST ||
-     (advances[tech].req[0]==A_LAST && advances[tech].req[1]==A_LAST))  {
+     (advance_required(tech, AR_ONE)==A_LAST
+     && advance_required(tech, AR_TWO)==A_LAST))  {
     sz_strlcpy(label, _("Removed"));
     bg=TREE_NODE_REMOVED_TECH_BG;
     l=XtVaCreateManagedWidget("treenode", commandWidgetClass, 
@@ -278,8 +282,8 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
   }
   
   my_snprintf(label, sizeof(label),
-	      "%s:%d", advance_name_translation(tech),
-	      num_unknown_techs_for_goal(game.player_ptr, tech));
+	      "%s:%d", advance_name_translation(advance_by_number(tech)),
+	      num_unknown_techs_for_goal(client.conn.playing, tech));
 
   if(parent) {
     l=XtVaCreateManagedWidget("treenode", 
@@ -305,10 +309,10 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
 
   
   if(--levels>0) {
-    if(advances[tech].req[0]!=A_NONE)
-      create_tech_tree(tree, l, advances[tech].req[0], levels);
-    if(advances[tech].req[1]!=A_NONE)
-      create_tech_tree(tree, l, advances[tech].req[1], levels);
+    if(advance_required(tech, AR_ONE)!=A_NONE)
+      create_tech_tree(tree, l, advance_required(tech, AR_ONE), levels);
+    if(advance_required(tech, AR_TWO)!=A_NONE)
+      create_tech_tree(tree, l, advance_required(tech, AR_TWO), levels);
   }
   
   
@@ -716,18 +720,18 @@ static void create_help_page(enum help_page_type type)
 ...
 **************************************************************************/
 static void help_update_improvement(const struct help_item *pitem,
-				    char *title, int which)
+				    char *title)
 {
   char buf[64000];
+  struct impr_type *imp = find_improvement_by_translated_name(title);
   
   create_help_page(HELP_IMPROVEMENT);
-  
-  if (which<game.control.num_impr_types) {
-    struct impr_type *imp = improvement_by_number(which);
-    int i;
-    char req_buf[512];
 
-    sprintf(buf, "%d ", impr_build_shield_cost(which));
+  if (imp  &&  !is_great_wonder(imp)) {
+    char req_buf[512];
+    int i;
+
+    sprintf(buf, "%d ", impr_build_shield_cost(imp));
     xaw_set_label(help_improvement_cost_data, buf);
     sprintf(buf, "%d ", imp->upkeep);
     xaw_set_label(help_improvement_upkeep_data, buf);
@@ -735,7 +739,7 @@ static void help_update_improvement(const struct help_item *pitem,
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
-		    advance_name_translation(imp->tech_req));
+		    advance_name_translation(advance_by_number(imp->tech_req)));
     }*/
     
     /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
@@ -745,13 +749,13 @@ static void help_update_improvement(const struct help_item *pitem,
     i = 0;
     requirement_vector_iterate(&imp->reqs, preq) {
       xaw_set_label(help_improvement_req_data,
-		    get_req_source_text(&preq->source,
-					req_buf, sizeof(req_buf)));
+                    universal_name_translation(&preq->source,
+                                               req_buf, sizeof(req_buf)));
       i++;
       break;
     } requirement_vector_iterate_end;
 /*    create_tech_tree(help_tech_tree, 0,
-                     imp->req[0].source.value.tech, 3);
+                     imp->req[0].source.value.advance, 3);
 */
   } else {
     xaw_set_label(help_improvement_cost_data, "0 ");
@@ -760,7 +764,7 @@ static void help_update_improvement(const struct help_item *pitem,
     create_tech_tree(help_tech_tree, 0, A_LAST, 3);
   }
   set_title_topic(pitem);
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 }
   
@@ -768,26 +772,20 @@ static void help_update_improvement(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_wonder(const struct help_item *pitem,
-			       char *title, int which)
+			       char *title)
 {
   char buf[64000];
-  
+  struct impr_type *imp = find_improvement_by_translated_name(title);
+
   create_help_page(HELP_WONDER);
 
-  if (which<game.control.num_impr_types) {
-    struct impr_type *imp = improvement_by_number(which);
-    int i;
+  if (imp  &&  is_great_wonder(imp)) {
     char req_buf[512];
+    int i;
+    struct advance *vap;
 
-    sprintf(buf, "%d ", impr_build_shield_cost(which));
+    sprintf(buf, "%d ", impr_build_shield_cost(imp));
     xaw_set_label(help_improvement_cost_data, buf);
-    /*if (imp->tech_req == A_LAST) {
-      xaw_set_label(help_improvement_req_data, _("(Never)"));
-    } else {
-      xaw_set_label(help_improvement_req_data,
-		    advance_name_translation(imp->tech_req));
-    }*/
-
      /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
       * Currently it's limited to 1 req but this code is partially prepared
       * to be extended.  Remember MAX_NUM_REQS is a compile-time
@@ -795,30 +793,31 @@ static void help_update_wonder(const struct help_item *pitem,
     i = 0;
     requirement_vector_iterate(&imp->reqs, preq) {
       xaw_set_label(help_improvement_req_data,
-                   get_req_source_text(&preq->source,
-                                       req_buf, sizeof(req_buf)));
+                    universal_name_translation(&preq->source,
+                                               req_buf, sizeof(req_buf)));
       i++;
       break;
     } requirement_vector_iterate_end;
 
-    if (tech_exists(imp->obsolete_by)) {
+    vap = valid_advance(imp->obsolete_by);
+    if (vap) {
       xaw_set_label(help_wonder_obsolete_data,
-		    advance_name_translation(imp->obsolete_by));
+		    advance_name_translation(vap));
     } else {
       xaw_set_label(help_wonder_obsolete_data, _("(Never)"));
     }
 /*    create_tech_tree(help_tech_tree, 0, 
-                     imp->req[0].source.value.tech, 3);
+                     imp->req[0].source.value.advance, 3);
 */
   } else {
     /* can't find wonder */
     xaw_set_label(help_improvement_cost_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
     xaw_set_label(help_wonder_obsolete_data, _("None"));
-    create_tech_tree(help_tech_tree, 0, game.control.num_tech_types, 3); 
+    create_tech_tree(help_tech_tree, 0, advance_count(), 3); 
   }
   set_title_topic(pitem);
-  helptext_building(buf, sizeof(buf), which, pitem->text);
+  helptext_building(buf, sizeof(buf), client.conn.playing, pitem->text, imp);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 }
 
@@ -826,14 +825,15 @@ static void help_update_wonder(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_unit_type(const struct help_item *pitem,
-				  char *title, struct unit_type *punittype)
+				  char *title)
 {
   char buf[64000];
+  struct unit_type *punittype = find_unit_type_by_translated_name(title);
   
   create_help_page(HELP_UNIT);
   if (punittype) {
 /*    struct unit_type *punittype = utype_by_number(i);*/
-    sprintf(buf, "%d ", unit_build_shield_cost(punittype));
+    sprintf(buf, "%d ", utype_build_shield_cost(punittype));
     xaw_set_label(help_unit_cost_data, buf);
     sprintf(buf, "%d ", punittype->attack_strength);
     xaw_set_label(help_unit_attack_data, buf);
@@ -849,22 +849,22 @@ static void help_update_unit_type(const struct help_item *pitem,
     xaw_set_label(help_unit_visrange_data, buf);
     xaw_set_label(help_unit_upkeep_data,
 		  helptext_unit_upkeep_str(punittype));
-    if (punittype->tech_requirement == A_LAST) {
+    if (A_NEVER == punittype->require_advance) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
-		    advance_name_for_player(game.player_ptr,
-				  punittype->tech_requirement));
+		    advance_name_for_player(client.conn.playing,
+				  advance_number(punittype->require_advance)));
     }
-    create_tech_tree(help_tech_tree, 0, punittype->tech_requirement, 3);
-    if (punittype->obsoleted_by == U_NOT_OBSOLETED) {
+    create_tech_tree(help_tech_tree, 0, advance_number(punittype->require_advance), 3);
+    if (U_NOT_OBSOLETED == punittype->obsoleted_by) {
       xaw_set_label(help_wonder_obsolete_data, _("None"));
     } else {
       xaw_set_label(help_wonder_obsolete_data,
 		    utype_name_translation(punittype->obsoleted_by));
     }
     /* add text for transport_capacity, fuel, and flags: */
-    helptext_unit(buf, sizeof(buf), punittype, pitem->text);
+    helptext_unit(buf, sizeof(buf), client.conn.playing, pitem->text, punittype);
     XtVaSetValues(help_text, XtNstring, buf, NULL);
   }
   else {
@@ -876,7 +876,7 @@ static void help_update_unit_type(const struct help_item *pitem,
     xaw_set_label(help_unit_hp_data, "0 ");
     xaw_set_label(help_unit_visrange_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
-    create_tech_tree(help_tech_tree, 0, game.control.num_tech_types, 3);
+    create_tech_tree(help_tech_tree, 0, advance_count(), 3);
     xaw_set_label(help_wonder_obsolete_data, _("None"));
     XtVaSetValues(help_text, XtNstring, pitem->text, NULL);
   }
@@ -887,64 +887,72 @@ static void help_update_unit_type(const struct help_item *pitem,
 /**************************************************************************
 ...
 **************************************************************************/
-static void help_update_tech(const struct help_item *pitem, char *title, int i)
+static void help_update_tech(const struct help_item *pitem, char *title)
 {
   char buf[4096];
-  int j;
+  int i;
+  struct advance *padvance = find_advance_by_translated_name(title);
 
   create_help_page(HELP_TECH);
   set_title_topic(pitem);
 
-  if (!is_future_tech(i)) {
+  if (padvance  &&  !is_future_tech(i = advance_number(padvance))) {
     create_tech_tree(help_tech_tree, 0, i, 3);
-    helptext_tech(buf, sizeof(buf), i, pitem->text);
+    helptext_advance(buf, sizeof(buf), client.conn.playing, pitem->text, i);
 
-    impr_type_iterate(impr_t) {
-      /*if(i==improvement_types[j].tech_req) 
+    improvement_iterate(pimprove) {
+      /*if (i == j->tech_req) 
 	sprintf(buf+strlen(buf), _("Allows %s.\n"),
-		improvement_types[j].name);
+		improvement_name_translation(j));
       */
 
        /* FIXME: need a more general mechanism for this, since this
         * helptext needs to be shown in all possible req source types. */
-      requirement_vector_iterate(&improvement_by_number(impr_t)->reqs, preq) {
-	if (preq->source.type == REQ_BUILDING
-	    && preq->source.value.building == i) {
+      requirement_vector_iterate(&pimprove->reqs, preq) {
+	if (VUT_IMPROVEMENT == preq->source.kind
+	    && preq->source.value.building == pimprove) {
 	  sprintf(buf+strlen(buf), _("Allows %s.\n"),
-		  improvement_name_translation(impr_t));
+		  improvement_name_translation(pimprove));
         }
       } requirement_vector_iterate_end;
-      if (i == improvement_by_number(impr_t)->obsolete_by)
+      if (padvance == pimprove->obsolete_by)
 	sprintf(buf+strlen(buf), _("Obsoletes %s.\n"),
-		improvement_name_translation(impr_t));
-    } impr_type_iterate_end;
+		improvement_name_translation(pimprove));
+    } improvement_iterate_end;
 
     unit_type_iterate(punittype) {
-      if (i == punittype->tech_requirement) 
-	sprintf(buf + strlen(buf), _("Allows %s.\n"),
-	        utype_name_translation(punittype));
+      if (padvance != punittype->require_advance) {
+	continue;
+      }
+      sprintf(buf + strlen(buf), _("Allows %s.\n"),
+	      utype_name_translation(punittype));
     } unit_type_iterate_end;
 
-    for (j = 0; j < game.control.num_tech_types; j++) {
-      if(i==advances[j].req[0]) {
-	if(advances[j].req[1]==A_NONE)
+    advance_iterate(A_NONE, ptest) {
+      if (padvance == advance_requires(ptest, AR_ONE)) {
+	if (advance_by_number(A_NONE) == advance_requires(ptest, AR_TWO))
 	  sprintf(buf+strlen(buf), _("Allows %s.\n"), 
-		  advance_name_translation(j));
+		  advance_name_translation(ptest));
 	else
 	  sprintf(buf+strlen(buf), _("Allows %s (with %s).\n"), 
-		  advance_name_translation(j),
-		  advance_name_translation(advances[j].req[1]));
+		  advance_name_translation(ptest),
+		  advance_name_translation(advance_requires(ptest, AR_TWO)));
       }
-      if(i==advances[j].req[1]) {
+      if (padvance == advance_requires(ptest, AR_TWO)) {
 	sprintf(buf+strlen(buf), _("Allows %s (with %s).\n"), 
-		advance_name_translation(j),
-		advance_name_translation(advances[j].req[0]));
+		advance_name_translation(ptest),
+		advance_name_translation(advance_requires(ptest, AR_ONE)));
       }
-    }
+    } advance_iterate_end;
     if (strlen(buf)) strcat(buf, "\n");
+/*
+    if (advances[i].helptext) {
+      sprintf(buf+strlen(buf), "%s\n", _(advances[i].helptext));
+    }
+*/
   } else {
     create_help_page(HELP_TECH);
-    create_tech_tree(help_tech_tree, 0, game.control.num_tech_types, 3);
+    create_tech_tree(help_tech_tree, 0, advance_count(), 3);
     strcpy(buf, pitem->text);
   }
   wordwrap_string(buf, 68);
@@ -955,14 +963,15 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 ...
 **************************************************************************/
 static void help_update_terrain(const struct help_item *pitem,
-				char *title, struct terrain *pterrain)
+				char *title)
 {
   char buf[4096];
+  struct terrain *pterrain = find_terrain_by_translated_name(title);
 
   create_help_page(HELP_TERRAIN);
   set_title_topic(pitem);
 
-  helptext_terrain(buf, sizeof(buf), pterrain, pitem->text);
+  helptext_terrain(buf, sizeof(buf), client.conn.playing, pitem->text, pterrain);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 
   if (pterrain) {
@@ -1046,15 +1055,15 @@ static void help_update_terrain(const struct help_item *pitem,
   This is currently just a text page, with special text:
 **************************************************************************/
 static void help_update_government(const struct help_item *pitem,
-				   char *title,
-				   struct government *pgovernment)
+				   char *title)
 {
   char buf[4096];
+  struct government *pgovernment = find_government_by_translated_name(title);
 
   if (!pgovernment) {
     strcat(buf, pitem->text);
   } else {
-    helptext_government(buf, sizeof(buf), pgovernment, pitem->text);
+    helptext_government(buf, sizeof(buf), client.conn.playing, pitem->text, pgovernment);
   }
   create_help_page(HELP_TEXT);
   set_title_topic(pitem);
@@ -1066,7 +1075,6 @@ static void help_update_government(const struct help_item *pitem,
 **************************************************************************/
 static void help_update_dialog(const struct help_item *pitem)
 {
-  int i;
   char *top;
 
   /* figure out what kind of item is required for pitem ingo */
@@ -1077,26 +1085,22 @@ static void help_update_dialog(const struct help_item *pitem)
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
-    i = find_improvement_by_translated_name(top);
-    if(i!=B_LAST && is_great_wonder(i)) i = B_LAST;
-    help_update_improvement(pitem, top, i);
+    help_update_improvement(pitem, top);
     break;
   case HELP_WONDER:
-    i = find_improvement_by_translated_name(top);
-    if(i!=B_LAST && !is_great_wonder(i)) i = B_LAST;
-    help_update_wonder(pitem, top, i);
+    help_update_wonder(pitem, top);
     break;
   case HELP_UNIT:
-    help_update_unit_type(pitem, top, find_unit_type_by_translated_name(top));
+    help_update_unit_type(pitem, top);
     break;
   case HELP_TECH:
-    help_update_tech(pitem, top, find_advance_by_translated_name(top));
+    help_update_tech(pitem, top);
     break;
   case HELP_TERRAIN:
-    help_update_terrain(pitem, top, find_terrain_by_translated_name(top));
+    help_update_terrain(pitem, top);
     break;
   case HELP_GOVERNMENT:
-    help_update_government(pitem, top, find_government_by_translated_name(top));
+    help_update_government(pitem, top);
     break;
   case HELP_TEXT:
   default:
@@ -1152,10 +1156,10 @@ static void help_tree_node_callback(Widget w, XtPointer client_data,
   size_t tech=(size_t)client_data;
   
   if(!help_tree_destroy_children(w)) {
-    if(advances[tech].req[0]!=A_NONE)
-      create_tech_tree(help_tech_tree, w, advances[tech].req[0], 1);
-    if(advances[tech].req[1]!=A_NONE)
-      create_tech_tree(help_tech_tree, w, advances[tech].req[1], 1);
+    if(advance_required(tech, AR_ONE)!=A_NONE)
+      create_tech_tree(help_tech_tree, w, advance_required(tech, AR_ONE), 1);
+    if(advance_required(tech, AR_TWO)!=A_NONE)
+      create_tech_tree(help_tech_tree, w, advance_required(tech, AR_TWO), 1);
   }
   
 }
