@@ -24,6 +24,7 @@ enum MenuIndex {
   MENU_VIEW,
   MENU_ORDER,
   MENU_REPORT,
+  MENU_EDITOR,
   MENU_HELP,
 
   MENU_LAST
@@ -36,8 +37,7 @@ enum MenuID {
   MENU_GAME_OPTIONS,
   MENU_GAME_MSG_OPTIONS,
   MENU_GAME_SAVE_SETTINGS,
-  MENU_GAME_SERVER_OPTIONS1,
-  MENU_GAME_SERVER_OPTIONS2,
+  MENU_GAME_SERVER_OPTIONS,
   MENU_GAME_OUTPUT_LOG,
   MENU_GAME_CLEAR_OUTPUT,
   MENU_GAME_DISCONNECT,
@@ -97,9 +97,10 @@ enum MenuID {
   MENU_ORDER_RETURN,
   MENU_ORDER_DISBAND,
   MENU_ORDER_BUILD_WONDER,
-  MENU_ORDER_TRADEROUTE,
+  MENU_ORDER_TRADE_ROUTE,
   MENU_ORDER_DIPLOMAT_DLG,
   MENU_ORDER_NUKE,
+  MENU_ORDER_SELECT_SAME_TYPE,
   MENU_ORDER_WAIT,
   MENU_ORDER_DONE,
 
@@ -114,11 +115,19 @@ enum MenuID {
   MENU_REPORT_DEMOGRAPHIC,
   MENU_REPORT_SPACESHIP,
 
+  MENU_EDITOR_TOGGLE,
+  MENU_EDITOR_TOOLS,
+
   MENU_HELP_LANGUAGES,
   MENU_HELP_CONNECTING,
   MENU_HELP_CONTROLS,
+  MENU_HELP_CMA,
   MENU_HELP_CHATLINE,
+  MENU_HELP_WORKLIST_EDITOR,
+  MENU_HELP_OVERVIEW,
   MENU_HELP_PLAYING,
+  MENU_HELP_ECONOMY,
+  MENU_HELP_CITIES,
   MENU_HELP_IMPROVEMENTS,
   MENU_HELP_UNITS,
   MENU_HELP_COMBAT,
@@ -127,8 +136,9 @@ enum MenuID {
   MENU_HELP_TERRAIN,
   MENU_HELP_WONDERS,
   MENU_HELP_GOVERNMENT,
-  MENU_HELP_HAPPINESS,
+  MENU_HELP_DIPLOMACY,
   MENU_HELP_SPACE_RACE,
+  MENU_HELP_RULESET,
   MENU_HELP_COPYING,
   MENU_HELP_ABOUT,
 
