@@ -14,33 +14,10 @@
 #define FC__UNIT_H
 
 #include "fc_types.h"
+#include "base.h"
 #include "terrain.h"		/* enum tile_special_type */
 #include "unittype.h"
-
-#define BARBARIAN_LIFE    5
-
-/* Changing this enum will break savegame and network compatability. */
-enum unit_activity {
-  ACTIVITY_IDLE = 0,
-  ACTIVITY_POLLUTION = 1,
-  ACTIVITY_ROAD = 2,
-  ACTIVITY_MINE = 3,
-  ACTIVITY_IRRIGATE = 4,
-  ACTIVITY_FORTIFIED = 5,
-  ACTIVITY_FORTRESS = 6,
-  ACTIVITY_SENTRY = 7,
-  ACTIVITY_RAILROAD = 8,
-  ACTIVITY_PILLAGE = 9,
-  ACTIVITY_GOTO = 10,
-  ACTIVITY_EXPLORE = 11,
-  ACTIVITY_TRANSFORM = 12,
-  ACTIVITY_UNKNOWN = 13,		/* savegame compatability. */
-  ACTIVITY_AIRBASE = 14,
-  ACTIVITY_FORTIFYING = 15,
-  ACTIVITY_FALLOUT = 16,
-  ACTIVITY_PATROL_UNUSED = 17,		/* savegame compatability. */
-  ACTIVITY_LAST   /* leave this one last */
-};
+#include "vision.h"
 
 /* Changing this enum will break network compatability. */
 enum unit_orders {
@@ -50,7 +27,7 @@ enum unit_orders {
   ORDER_BUILD_CITY = 3,
   ORDER_DISBAND = 4,
   ORDER_BUILD_WONDER = 5,
-  ORDER_TRADEROUTE = 6,
+  ORDER_TRADE_ROUTE = 6,
   ORDER_HOMECITY = 7,
   /* and plenty more for later... */
   ORDER_LAST
@@ -101,7 +78,8 @@ enum unit_move_result {
   MR_BAD_MAP_POSITION,
   MR_DESTINATION_OCCUPIED_BY_NON_ALLIED_CITY,
   MR_DESTINATION_OCCUPIED_BY_NON_ALLIED_UNIT,
-  MR_NO_SEA_TRANSPORTER_CAPACITY,
+  MR_NO_TRANSPORTER_CAPACITY,
+  MR_TRIREME,
 };
 
 enum add_build_city_result {
@@ -152,21 +130,24 @@ struct unit_ai {
 struct unit_order {
   enum unit_orders order;
   enum unit_activity activity;  /* Only valid for ORDER_ACTIVITY. */
+  Base_type_id base;            /* Only valid for activity ACTIVITY_BASE */
   enum direction8 dir;          /* Only valid for ORDER_MOVE. */
 };
 
 struct unit {
-  struct unit_type *utype;
-  int id;
-  struct player *owner; /* Cannot be NULL. */
+  struct unit_type *utype; /* Cannot be NULL. */
   struct tile *tile;
+  struct player *owner; /* Cannot be NULL. */
+  int id;
   int homecity;
+
+  int upkeep[O_LAST]; /* unit upkeep with regards to the homecity */
+
   int moves_left;
   int hp;
   int veteran;
-  int unhappiness;
-  int upkeep[O_MAX];
   int fuel;
+  int birth_turn;
   struct unit_ai ai;
   enum unit_activity activity;
   struct tile *goto_tile; /* May be NULL. */
@@ -177,6 +158,7 @@ struct unit {
   int activity_count;
 
   enum tile_special_type activity_target;
+  Base_type_id           activity_base;
   enum unit_focus_status focus_status;
   int ord_map, ord_city;
   /* ord_map and ord_city are the order index of this unit in tile.units
@@ -206,7 +188,7 @@ struct unit {
     int color_index;
   } client;
   struct {
-    struct vision *vision; /* See explanation in maphand.h. */
+    struct vision *vision;
   } server;
 
   bool has_orders;
@@ -218,11 +200,6 @@ struct unit {
   } orders;
 };
 
-#define SINGLE_MOVE 3
-#define MOVE_COST_RIVER 1
-#define MOVE_COST_RAIL 0
-#define MOVE_COST_ROAD 1
-
 bool is_real_activity(enum unit_activity activity);
 
 /* Iterates over the types of unit activity. */
@@ -248,10 +225,11 @@ bool is_diplomat_action_available(const struct unit *pdiplomat,
 bool unit_can_help_build_wonder(const struct unit *punit,
 				const struct city *pcity);
 bool unit_can_help_build_wonder_here(const struct unit *punit);
-bool unit_can_est_traderoute_here(const struct unit *punit);
+bool unit_can_est_trade_route_here(const struct unit *punit);
 bool unit_can_airlift_to(const struct unit *punit, const struct city *pcity);
 bool unit_has_orders(const struct unit *punit);
 
+bool could_unit_load(const struct unit *pcargo, const struct unit *ptrans);
 bool can_unit_load(const struct unit *punit, const struct unit *ptrans);
 bool can_unit_unload(const struct unit *punit, const struct unit *ptrans);
 bool can_unit_paradrop(const struct unit *punit);
@@ -265,15 +243,22 @@ bool can_unit_do_activity(const struct unit *punit,
 			  enum unit_activity activity);
 bool can_unit_do_activity_targeted(const struct unit *punit,
 				   enum unit_activity activity,
-				   enum tile_special_type target);
+				   enum tile_special_type target,
+                                   Base_type_id base);
 bool can_unit_do_activity_targeted_at(const struct unit *punit,
 				      enum unit_activity activity,
 				      enum tile_special_type target,
-				      const struct tile *ptile);
+				      const struct tile *ptile,
+                                      Base_type_id base);
+bool can_unit_do_activity_base(const struct unit *punit,
+                               Base_type_id base);
 void set_unit_activity(struct unit *punit, enum unit_activity new_activity);
 void set_unit_activity_targeted(struct unit *punit,
 				enum unit_activity new_activity,
-				enum tile_special_type new_target);
+				enum tile_special_type new_target,
+                                Base_type_id base);
+void set_unit_activity_base(struct unit *punit,
+                            Base_type_id base);
 int get_activity_rate(const struct unit *punit);
 int get_activity_rate_this_turn(const struct unit *punit);
 int get_turns_for_activity_at(const struct unit *punit,
@@ -283,6 +268,7 @@ bool can_unit_do_autosettlers(const struct unit *punit);
 bool is_unit_activity_on_tile(enum unit_activity activity,
 			      const struct tile *ptile);
 bv_special get_unit_tile_pillage_set(const struct tile *ptile);
+bv_bases get_unit_tile_pillage_base_set(const struct tile *ptile);
 bool is_attack_unit(const struct unit *punit);
 bool is_military_unit(const struct unit *punit);           /* !set !dip !cara */
 bool is_diplomat_unit(const struct unit *punit);
@@ -291,7 +277,8 @@ bool is_square_threatened(const struct player *pplayer,
 bool is_field_unit(const struct unit *punit);              /* ships+aero */
 bool is_hiding_unit(const struct unit *punit);
 #define COULD_OCCUPY(punit) \
-  ((is_ground_unit(punit) || is_heli_unit(punit)) && is_military_unit(punit))
+  (uclass_has_flag(unit_class(punit), UCF_CAN_OCCUPY_CITY) \
+   && is_military_unit(punit))
 bool can_unit_add_to_city (const struct unit *punit);
 bool can_unit_build_city (const struct unit *punit);
 bool can_unit_add_or_build_city (const struct unit *punit);
@@ -299,17 +286,15 @@ enum add_build_city_result test_unit_add_or_build_city(const struct unit *
 						       punit);
 bool kills_citizen_after_attack(const struct unit *punit);
 
+struct astring; /* Forward declaration. */
+void unit_activity_astr(const struct unit *punit, struct astring *astr);
+void unit_upkeep_astr(const struct unit *punit, struct astring *astr);
 const char *unit_activity_text(const struct unit *punit);
-int ground_unit_transporter_capacity(const struct tile *ptile,
-				     const struct player *pplayer);
+
 int get_transporter_capacity(const struct unit *punit);
-bool is_ground_units_transport(const struct unit *punit);
-int missile_carrier_capacity(const struct tile *ptile,
-			     const struct player *pplayer);
-int airunit_carrier_capacity(const struct tile *ptile,
-			     const struct player *pplayer);
 
 struct player *unit_owner(const struct unit *punit);
+struct tile *unit_tile(const struct unit *punit);
 
 struct unit *is_allied_unit_tile(const struct tile *ptile,
 				 const struct player *pplayer);
@@ -319,13 +304,8 @@ struct unit *is_non_allied_unit_tile(const struct tile *ptile,
 				     const struct player *pplayer);
 struct unit *is_non_attack_unit_tile(const struct tile *ptile,
 				     const struct player *pplayer);
-
-int unit_loss_pct(const struct player *pplayer, const struct tile *ptile,
-		  const struct unit *punit);
-int base_trireme_loss_pct(const struct player *pplayer,
-			  const struct unit *punit);
-int base_unsafe_terrain_loss_pct(const struct player *pplayer,
-				 const struct unit *punit);
+struct unit *unit_occupies_tile(const struct tile *ptile,
+				const struct player *pplayer);
 
 bool is_my_zoc(const struct player *unit_owner, const struct tile *ptile);
 bool unit_being_aggressive(const struct unit *punit);
@@ -337,15 +317,20 @@ struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
                                  struct unit_type *punittype,
 				 int veteran_level);
 void destroy_unit_virtual(struct unit *punit);
+bool unit_is_virtual(const struct unit *punit);
 void free_unit_orders(struct unit *punit);
 
 int get_transporter_occupancy(const struct unit *ptrans);
-struct unit *find_transporter_for_unit(const struct unit *pcargo,
-				       const struct tile *ptile);
+struct unit *find_transporter_for_unit(const struct unit *pcargo);
 
 enum unit_upgrade_result test_unit_upgrade(const struct unit *punit,
 					   bool is_free);
 enum unit_upgrade_result get_unit_upgrade_info(char *buf, size_t bufsz,
 					       const struct unit *punit);
+bool is_losing_hp(const struct unit *punit);
+bool unit_type_is_losing_hp(const struct player *pplayer,
+                            const struct unit_type *punittype);
+
+bool unit_alive(int id);
 
 #endif  /* FC__UNIT_H */
