@@ -14,7 +14,9 @@
 #ifndef FC__BARBARIAN_H
 #define FC__BARBARIAN_H
 
-struct player;
+#include "shared.h"		/* bool type */
+
+#include "fc_types.h"
 
 #define MIN_UNREST_DIST   5
 #define MAX_UNREST_DIST   8
@@ -23,16 +25,13 @@ struct player;
 #define UPRISE_CIV_MORE  30
 #define UPRISE_CIV_MOST  50
 
-#define LAND_BARBARIAN    1
-#define SEA_BARBARIAN     2
-
-#define BARBARIAN_LIFE    5
-
 #define MAP_FACTOR     2000  /* adjust this to get a good uprising frequency */
 
-int unleash_barbarians(struct player* vict, int x, int y);
+#define BARBARIAN_MIN_LIFESPAN 5
+
+bool unleash_barbarians(struct tile *ptile);
 void summon_barbarians(void);
-int is_land_barbarian(struct player *pplayer);
-int is_sea_barbarian(struct player *pplayer);
+bool is_land_barbarian(struct player *pplayer);
+bool is_sea_barbarian(struct player *pplayer);
 
 #endif  /* FC__BARBARIAN_H */
