@@ -203,9 +203,13 @@ void log_set_level(int level)
 /**************************************************************************
 Adjust the callback function after initial log_init().
 **************************************************************************/
-void log_set_callback(log_callback_fn callback)
+log_callback_fn log_set_callback(log_callback_fn callback)
 {
-  log_callback=callback;
+  log_callback_fn old = log_callback;
+
+  log_callback = callback;
+
+  return old;
 }
 
 /**************************************************************************
@@ -271,7 +275,7 @@ void vreal_freelog(int level, const char *message, va_list ap)
     FILE *fs;
 
     if (log_filename) {
-      if(!(fs=fopen(log_filename, "a"))) {
+      if(!(fs=fc_fopen(log_filename, "a"))) {
 	fc_fprintf(stderr,
 		   _("Couldn't open logfile: %s for appending \"%s\".\n"), 
 		   log_filename, message);
